#!/usr/bin/env python
'''
This file contains Python command example for setting and reading atcam counters.
It can be used as a debugging utility for reading the atcam & erp counters.
'''
import sys
import errno
import pdb
import time
from python_sdk_api.sx_api import *
from python_sdk_api.sxd_api import *
from test_infra_common import *
import argparse

######################################################
#    defines
######################################################
SWID = 0
DEVICE_ID = 1

ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

parser = argparse.ArgumentParser(description='Set/Get atcam region & erp debug counters',
                                 formatter_class=argparse.ArgumentDefaultsHelpFormatter)
parser.add_argument('--region_id', default=0xffff, type=lambda x: int(x, 0), help='The region ids for which to apply counters. Mandatory for set operation.')
parser.add_argument('--region_id_mask', default=0, type=lambda x: int(x, 0), help='Bitmask: 0 compare region_id relevant bit, 1 ignore region_id relevant bit.')
parser.add_argument('--erp_mask', default=0xffff, type=lambda x: int(x, 0), help='Bitmask: Erp IDs for which to apply counters. Each bit represent an erp id. If region_id_bitmask !=0 all erps will be counted.')
parser.add_argument('--ctcam', action='store_true', help='Whether to enable or disable counters on the ctcam.')
parser.add_argument('--clear', action='store_true', help='Whether to clear the counters after read. If region_id_bitmask !=0 ctcam will be counted.')
args = parser.parse_args()


def main():

    print "[+] opening sdk"
    rc, handle = sx_api_open(None)
    print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

    # If region id is specified we want to do a set
    if (args.region_id != 0xffff):
        # Set the region counters
        region_cmd_info = sx_dbg_atcam_cmd_info_t()
        region_cmd_info.type = SX_DBG_ATCAM_CMD_REGION_COUNTERS_SET_E
        region_cmd_info.data.region_counters_set.region_id = args.region_id
        region_cmd_info.data.region_counters_set.region_id_bitmask = args.region_id_mask

        region_cmd_info_p = new_sx_dbg_atcam_cmd_info_t_p()
        sx_dbg_atcam_cmd_info_t_p_assign(region_cmd_info_p, region_cmd_info)
        rc = sx_api_dbg_atcam(handle, region_cmd_info_p)
        assert rc == SX_STATUS_SUCCESS, "Failed to set region atcam counters."

        # Set the erp counters
        erp_cmd_info = sx_dbg_atcam_cmd_info_t()
        erp_cmd_info.type = SX_DBG_ATCAM_CMD_ERP_COUNTERS_SET_E
        erp_cmd_info.data.erp_counters_set.region_id = args.region_id
        erp_cmd_info.data.erp_counters_set.region_id_bitmask = args.region_id_mask
        erp_cmd_info.data.erp_counters_set.erp_bitmask = args.erp_mask
        if not args.ctcam:
            erp_cmd_info.data.erp_counters_set.ctcam = 0
        else:
            erp_cmd_info.data.erp_counters_set.ctcam = 1

        erp_cmd_info_p = new_sx_dbg_atcam_cmd_info_t_p()
        sx_dbg_atcam_cmd_info_t_p_assign(erp_cmd_info_p, erp_cmd_info)
        rc = sx_api_dbg_atcam(handle, erp_cmd_info_p)
        assert rc == SX_STATUS_SUCCESS, "Failed to set region erp counters."

        print "NOTE: Counters cleared after setting the region"
    else:
        # Read the region counters
        region_cmd_info = sx_dbg_atcam_cmd_info_t()
        region_cmd_info.type = SX_DBG_ATCAM_CMD_REGION_COUNTERS_READ_E
        if args.clear:
            region_cmd_info.data.region_counters_read.clear = 1
        else:
            region_cmd_info.data.region_counters_read.clear = 0

        region_cmd_info_p = new_sx_dbg_atcam_cmd_info_t_p()
        sx_dbg_atcam_cmd_info_t_p_assign(region_cmd_info_p, region_cmd_info)
        rc = sx_api_dbg_atcam(handle, region_cmd_info_p)
        assert rc == SX_STATUS_SUCCESS, "Failed to read region atcam counters."
        region_cmd_info = sx_dbg_atcam_cmd_info_t_p_value(region_cmd_info_p)

        # Read the erp counters
        erp_cmd_info = sx_dbg_atcam_cmd_info_t()
        erp_cmd_info.type = SX_DBG_ATCAM_CMD_ERP_COUNTERS_READ_E
        if args.clear:
            erp_cmd_info.data.erp_counters_read.clear = 1
        else:
            erp_cmd_info.data.erp_counters_read.clear = 0

        erp_cmd_info_p = new_sx_dbg_atcam_cmd_info_t_p()
        sx_dbg_atcam_cmd_info_t_p_assign(erp_cmd_info_p, erp_cmd_info)
        rc = sx_api_dbg_atcam(handle, erp_cmd_info_p)
        assert rc == SX_STATUS_SUCCESS, "Failed to read erp atcam counters."
        erp_cmd_info = sx_dbg_atcam_cmd_info_t_p_value(erp_cmd_info_p)

        print "=================================================="
        print "              REGIONS COUNTERS                    "
        print "=================================================="
        print "Lookup count                         : " + str(region_cmd_info.data.region_counters_read.lookup_count)
        print "Full lookup count (no cache hit)     : " + str(region_cmd_info.data.region_counters_read.full_lookup_count)
        print "=================================================="
        print "                ERP COUNTERS                      "
        print "=================================================="
        print "ERPs + CTCAM before any filtering    : " + str(erp_cmd_info.data.erp_counters_read.initial_count)
        print "ERPs active after BF (no CTCAM)      : " + str(erp_cmd_info.data.erp_counters_read.post_bf_count)
        print "ERPs + CTCAM after BF and prune      : " + str(erp_cmd_info.data.erp_counters_read.lookup_count)
        print "ERPs + CTCAM had any match:          : " + str(erp_cmd_info.data.erp_counters_read.any_match_count)
        print "ERPs or CTCAM final match            : " + str(erp_cmd_info.data.erp_counters_read.final_match_count)
        print "=================================================="

        if args.clear:
            print "NOTE: Counters cleared after read"

    print("Success.")

    sx_api_close(handle)


################################################################################
#                             Main                                             #
################################################################################
if __name__ == "__main__":
    main()
