#!/usr/bin/env python
'''
This file contains Python command example to set port rate (speed) value using sx_api_port_rate_set API.
Python commands syntax is very similar to the Switch SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.
Note that this example is supported only if switch was run using the same API and not sx_api_port_speed_admin_set.
'''
import sys
import errno
import sys
import colorsys
import argparse

from python_sdk_api.sx_api import *
from test_infra_common import *

valid_rates = ["100M", "1G", "10G", "25G", "40G", "50G", "50Gx1", "50Gx2",
               "100G", "100Gx2", "100Gx4", "200G", "200Gx4", "400G", "400Gx8", "auto"]


def help_cmd():
    print ("\n")
    print (sys.argv)
    print ("    Bad parameters!")
    print ("    Please use the following format to run example:")
    print ("    sx_api_port_rate_set.py --log_port=<PORT> --rate=<RATE> --force=False")
    print ("    or")
    print ("    sx_api_port_rate_set.py --log_port=<PORT_1, PORT_2>--rate=<RATE_1, RATE_2> --force=True")
    print ("    Valid speed values are " + str(valid_rates))
    print ("    Note that some of the speeds may not be supported by switch. Please check port rate capabilities.\n")
    sys.exit(errno.EACCES)


def validate_rate_args(rates_list):
    for rate in rates_list:
        if rate not in valid_rates:
            print ("Specified %s rate value is not valid." % (rate))
            help_cmd()


def parse_example_attributes(handle):

    parser = argparse.ArgumentParser(description='Port rate set example.')
    parser.add_argument('--log_port', nargs='+', type=str, help='logical port to change rate', default=['0x10100'])
    parser.add_argument('--rate', nargs='+', type=str, help='new port rate value', default=['10G'])
    parser.add_argument('--force', type=bool, help='Disable Auto-negotiation on phy. Only a single rate is allowed for this option', default=False)
    args = parser.parse_args()

    log_port = args.log_port[0].split(",")
    admin_rate = args.rate[0].split(",")
    force = args.force

    validate_rate_args(admin_rate)

    return log_port, admin_rate, force


def change_port_state(handle, log_port, admin_state):
    rc = sx_api_port_state_set(handle, log_port, admin_state)
    if rc != SX_STATUS_SUCCESS:
        print ("Error: failed to change port state, rc=%d." % (rc))
        sys.exit(errno.EACCES)


def enable_rate(rate, admin_rate):
    return (True if (rate in admin_rate) else False)


def run_example(handle, log_port, admin_rate, force):
    rate_bitmask_p = new_sx_port_rate_bitmask_t_p()

    # went throug the admin_rate list and enable appropriate values
    rate_bitmask_p.rate_100M = enable_rate("100M", admin_rate)
    rate_bitmask_p.rate_1G = enable_rate("1G", admin_rate)
    rate_bitmask_p.rate_10G = enable_rate("10G", admin_rate)
    rate_bitmask_p.rate_25G = enable_rate("25G", admin_rate)
    rate_bitmask_p.rate_40G = enable_rate("40G", admin_rate)
    rate_bitmask_p.rate_50G = enable_rate("50G", admin_rate)
    rate_bitmask_p.rate_50Gx1 = enable_rate("50Gx1", admin_rate)
    rate_bitmask_p.rate_50Gx2 = enable_rate("50Gx2", admin_rate)
    rate_bitmask_p.rate_100G = enable_rate("100G", admin_rate)
    rate_bitmask_p.rate_100Gx2 = enable_rate("100Gx2", admin_rate)
    rate_bitmask_p.rate_100Gx4 = enable_rate("100Gx4", admin_rate)
    rate_bitmask_p.rate_200G = enable_rate("200G", admin_rate)
    rate_bitmask_p.rate_200Gx4 = enable_rate("200Gx4", admin_rate)
    rate_bitmask_p.rate_400G = enable_rate("400G", admin_rate)
    rate_bitmask_p.rate_400Gx8 = enable_rate("400Gx8", admin_rate)
    rate_bitmask_p.rate_auto = enable_rate("auto", admin_rate)
    rate_bitmask_p.force = force

    for port in log_port:
        # convert string of log_port to appropriate integer value
        log_port = int(port, 0)

        change_port_state(handle, log_port, SX_PORT_ADMIN_STATUS_DOWN)

        rc = sx_api_port_rate_set(handle, log_port, rate_bitmask_p)
        if rc != SX_STATUS_SUCCESS:
            if rc == SX_STATUS_CMD_UNSUPPORTED:
                print("This API is not supported on this device (rc=%d)" % rc)
                sys.exit(0)
            print ("Error: failed to change %s port rate, rc=%d." % (port, rc))
            if rc == SX_STATUS_CMD_UNPERMITTED:
                print ("  Looks like switch was configured with sx_api_port_speed_admin_set API, so using sx_api_port_rate_set is not allowed !\n")
            sys.exit(errno.EACCES)

        change_port_state(handle, log_port, SX_PORT_ADMIN_STATUS_UP)

        print ("Admin rate value for %s log_port was changed to %s" % (port, admin_rate))



################################################
# Run the MAIN function
################################################
if __name__ == "__main__":

    print "[+] opening sdk"
    rc, handle = sx_api_open(None)
    print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))

    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

    log_port, admin_rate, force = parse_example_attributes(handle)

    # Run the main sx_api_port_rate_set API example with retrieved above parameters
    run_example(handle, log_port, admin_rate, force)

    sx_api_close(handle)

    print("Example execution is finished successfully.")
