#!/usr/bin/env python
'''
Python commands syntax is very similar to the Switch SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

This utility dumps the module power mode.
'''
import sys
import errno
import argparse
from python_sdk_api.sx_api import *
from test_infra_common import *
import python_sdk_api.sx_api as sx_api
from python_sdk_api.sxd_api import *

######################################################
#  power mode and reset utility functions.
######################################################


def mgmt_phy_mod_pwr_attr_get(handle, module_id, power_attr_type):
    sx_mgmt_phy_mod_pwr_attr_p = new_sx_mgmt_phy_mod_pwr_attr_t_p()
    sx_mgmt_phy_mod_pwr_attr = sx_mgmt_phy_mod_pwr_attr_t()
    sx_mgmt_phy_mod_pwr_attr.power_attr_type = power_attr_type
    sx_mgmt_phy_mod_pwr_attr_t_p_assign(sx_mgmt_phy_mod_pwr_attr_p, sx_mgmt_phy_mod_pwr_attr)
    try:
        rc = sx_mgmt_phy_mod_pwr_attr_get(handle, module_id, sx_mgmt_phy_mod_pwr_attr_p)
        assert SX_STATUS_SUCCESS == rc, "sx_mgmt_phy_mod_pwr_attr_get failed"
        sx_mgmt_phy_mod_pwr_attr = sx_mgmt_phy_mod_pwr_attr_t_p_value(sx_mgmt_phy_mod_pwr_attr_p)
        pwr_mode_attr = sx_mgmt_phy_mod_pwr_attr.pwr_mode_attr
        return pwr_mode_attr.admin_pwr_mode_e, pwr_mode_attr.oper_pwr_mode_e
    finally:
        delete_sx_mgmt_phy_mod_pwr_attr_t_p(sx_mgmt_phy_mod_pwr_attr_p)


##############################################################################
#  utility functions to set port state, get module port mapping, retrieve enum
##############################################################################

def del_dict_duplicate_values(dict):
    for key in dict.keys():
        if key.endswith("_MIN_E"):
            del dict[key]
        if key.endswith("_MAX_E"):
            del dict[key]
    return dict

def sx_api_get_enums_from_prefix(prefix):
    sx_api_str_enum_dict = {}
    for item in dir(sx_api):
        if item.startswith(prefix):
            sx_api_str_enum_dict[item] = getattr(sx_api, item)
    return sx_api_str_enum_dict


def get_enum_string_dict(enum_prefix):
    # dict with {string: enum_value}
    string_enum = sx_api_get_enums_from_prefix(enum_prefix)
    # dict with {enum_value: string}
    string_enum = del_dict_duplicate_values(string_enum)
    enum_string = dict((v, k) for k, v in string_enum.iteritems())
    return enum_string


def get_cable_type(port_list):
    cable_data_dict = {}
    cable_types_dict = {0: "UNIDENTIFIED_CABLE",
                        1: "ACTIVE_CABLE",
                        2: "OPTICAL_CABLE",
                        3: "PASSIVE_COPPER_CABLE",
                        4: "CABLE_UNPLUGGED"}

    localport = (port_list[0] & 0xFF00) >> SX_PORT_PHY_ID_OFFS

    pddr = ku_pddr_reg()
    pddr.local_port = localport
    pddr.page_select = PDDR_PAGE_OPT_MODULE_INFO_E

    meta = sxd_reg_meta_t()
    meta.dev_id = 1
    meta.swid = 0
    meta.access_cmd = SXD_ACCESS_CMD_GET
    sxd_access_reg_pddr(pddr, meta, 1, None, None)

    return cable_types_dict[pddr.response.module_info.cable_type]


###########################################################################################################

def generate_module_dump(handle, port_module_map, module_port_map):
    sxd_access_reg_init(0, None, SX_VERBOSITY_LEVEL_NONE)
    print ("=================================================================================================================")
    header = ["Module ID", "Panel", "Admin Power Mode", "Oper Power Mode", "Cable Type"]
    print "|%10s|%5s|%35s|%35s|%22s|" % (header[0], header[1], header[2], header[3], header[4])
    print ("=================================================================================================================")

    pwr_mode_enum_dict = get_enum_string_dict('SX_MGMT_PHY_MOD_PWR_MODE')
    for mod_id in sorted(module_port_map.keys()):
        admin_pwr_mode, oper_pwr_mode = mgmt_phy_mod_pwr_attr_get(handle, mod_id, SX_MGMT_PHY_MOD_PWR_ATTR_PWR_MODE_E)
        print ("|%10s|%5s|%35s|%35s|%22s|" % (mod_id, (mod_id + 1), pwr_mode_enum_dict[admin_pwr_mode],
                                              pwr_mode_enum_dict[oper_pwr_mode], get_cable_type(module_port_map[mod_id])))
    sxd_access_reg_deinit()


def run_example(handle, module_id):
    # construct the port module map.
    ports_attributes_list = ports_attributes_get(handle)
    port_module_map, module_port_map = get_port_module_mapping(ports_attributes_list, len(ports_attributes_list))

    if module_id == "all":
        generate_module_dump(handle, port_module_map, module_port_map)
    else:
        mod_id = int(module_id)
        if mod_id not in module_port_map.keys():
            print("Error: Invalid Module Id")
            sys.exit(0)
        admin_pwr_mode, oper_pwr_mode = mgmt_phy_mod_pwr_attr_get(handle, mod_id, SX_MGMT_PHY_MOD_PWR_ATTR_PWR_MODE_E)
        pwr_mode_enum_dict = get_enum_string_dict('SX_MGMT_PHY_MOD_PWR_MODE')
        print "Current power mode for module: [%d]" % (mod_id)
        print "Admin power mode: [%s]" % (pwr_mode_enum_dict[admin_pwr_mode])
        print "Operational power mode: [%s]" % (pwr_mode_enum_dict[oper_pwr_mode])


def parse_example_attributes():

    parser = argparse.ArgumentParser(description='Module power mode dump utility')
    parser.add_argument('--module_id', default='all', type=str, help="Module id")
    args = parser.parse_args()

    if len(sys.argv) < 1:
        print "Usage:"
        print "Example: Dump module power mode sx_mgmt_module_power_mode_dump.py --module_id all"
        print "Example: Dump module power mode sx_mgmt_module_power_mode_dump.py --module_id 1"
        sys.exit(0)

    module_id = args.module_id

    return module_id


################################################
# Run the MAIN function
################################################
if __name__ == "__main__":

    module_id = parse_example_attributes()

    print "[+] opening sdk"
    rc, handle = sx_api_open(None)
    print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

    # Run the  example with retrieved above parameters
    run_example(handle, module_id)

    sx_api_close(handle)
