#!/usr/bin/env python
'''
This file contains Python front end for "What just happened" lib feature and does the following with respective options
1. Enable WJH example backend with agent mode or manual mode.
2. Disable WJH example backend.
3. Enable drop reason group.
4. Disable drop reason group.
5. Fetch the dropped packets and print to console.
6. Fetch the dropped packets and save to pcap file or txt file for aggregation channel.

For default manual mode, wjh_example.py will exit after each command, and the basic flow could be
wjh_example.py --enable                            # enable the wjh_example_backend to start capture the dropped packets
wjh_example.py --disable_drop_reason_group=l1      # disable l1 drop reason group for example
wjh_example.py --print                             # print the captured the dropped packets to console
wjh_example.py --save                              # save the captured the dropped packets to file /tmp/wjh_raw.pcap or /tmp/wjh_agg.txt
wjh_example.py --disable                           # disable the wjh_example_backend to stop capture

For agent mode, wjh_example.py will not exit and wait for traffic, the flow could be
wjh_example.py --mode=agent --print                # enable the wjh_example_backend and wait for the dropped packets and print to console
OR
wjh_example.py --mode=agent --save                 # enable the wjh_example_backend and wait for the dropped packets and save to file
'''

import argparse
import subprocess
import socket
import time
import os
import sys
import signal
import pickle

######################################################
#    defines
######################################################
WJH_BACKEND_SCRIPT = 'wjh_example_backend.py'
WJH_DIR_SLASH = '/'
WJH_MONITOR_SERVER_PORT = 20000
WJH_TOOL_BUF_SIZE = 1024
WJH_SOCKET_TIMEOUT = 5
WJH_SERVER_UPTIME_WAIT_PERIOD = 2

# API to send/receive message to/from WJH example backend


def wjh_client_send(buf):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(WJH_SOCKET_TIMEOUT)
    server_address = ('localhost', WJH_MONITOR_SERVER_PORT)
    data = None
    try:
        sock.connect(server_address)
        sock.sendall(buf)
        data = sock.recv(WJH_TOOL_BUF_SIZE)
    except Exception:
        print("use --enable to enable the WJH example backend.")
    finally:
        sock.close()
        return data


def wjh_action_enable(cmd):
    path = os.path.dirname(sys.argv[0])
    subprocess.Popen(["python", path + WJH_DIR_SLASH + WJH_BACKEND_SCRIPT])
    time.sleep(WJH_SERVER_UPTIME_WAIT_PERIOD)
    data = wjh_client_send(cmd)
    if data:
        print('%s' % data)
    else:
        print("Failed to enable the WJH example backend - please try again")


def wjh_action_save(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to save the packets - please enable the WJH example"


def wjh_action_enable_group(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to enable the drop reason group - please enable the WJH example"


def wjh_action_disable_group(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to disable the drop reason group - please enable the WJH example"


def wjh_action_print(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to save the packets - please enable the WJH example"


def wjh_action_disable(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to disable the WJH example backend - please check status and try again"


def bandwidth_type(x):
    x = int(x)
    if x < 0 or x > 100:
        raise argparse.ArgumentTypeError("Bandwidth is 0 - 100")
    return x


def handler(signum, frame):
    print 'Ctrl+C pressed, exit...'
    sys.exit(0)


def wjh_action_bind_to_aggregation_channel(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to bind the drop reason group to aggregation channel - please enable the WJH example"


def wjh_action_bind_to_cyclic_channel(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to bind the drop reason group to cyclic channel - please enable the WJH example"


def wjh_action_bind_to_cyclic_and_aggregation_channel(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to bind the drop reason group to cyclic-and-aggregation channel - please enable the WJH example"


def wjh_action_set_aggregation_channel_aggregation_read_mode(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to set the aggregation read mode of aggregation channel - please enable the WJH example"


def wjh_action_set_cyclic_and_aggregation_channel_aggregation_read_mode(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to set the aggregation read mode of cyclic-and-aggregation channel - please enable the WJH example"


def wjh_action_set_channel_timestamp_source(cmd):
    data = wjh_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to set the user channel timestamp source - please enable the WJH example"


parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                 description='''WJH lib example front end does the following with respective options
1. Enable WJH example backend with agent mode or manual mode.
2. Disable WJH example backend.
3. Enable drop reason group.
4. Disable drop reason group.
5. Fetch the dropped packets and print to console.
6. Fetch the dropped packets and save to pcap file or txt file for aggregation channel.

For default manual mode, wjh_example.py will exit after each command, and the basic flow could be
wjh_example.py --enable                            # enable the wjh_example_backend to start capture the dropped packets
wjh_example.py --disable_drop_reason_group=l1      # disable l1 drop reason group for example
wjh_example.py --print                             # print the captured the dropped packets to console
wjh_example.py --save                              # save the captured the dropped packets to file /tmp/wjh_raw.pcap or /tmp/wjh_agg.txt
wjh_example.py --disable                           # disable the wjh_example_backend to stop capture

For agent mode, wjh_example.py will not exit and wait for traffic, the flow could be
wjh_example.py --enable --mode=agent --print                # enable the wjh_example_backend and wait for the dropped packets and print to console
OR
wjh_example.py --enable --mode=agent --save                 # enable the wjh_example_backend and wait for the dropped packets and save to file''')

parser.add_argument('--disable', action='store_true', help='disables the WJH example backend')
parser.add_argument('--enable', action='store_true', help='enables the WJH example backend')
parser.add_argument('--bandwidth', default=0, type=bandwidth_type, help='set pcie bandwidth percentage for WJH for enable option')
parser.add_argument('--mode', default='manual', choices=['agent', 'manual'], help='set WJH mode for enable option')
parser.add_argument('--aggregation_key_mode', default='streaming', choices=['streaming', 'detailed'], help='set WJH aggregation key mode for enable action')
parser.add_argument('--print', action='store_true', help='prints the WJH captured packets')
parser.add_argument('--save', action='store_true', help='saves the WJH captured packets')
parser.add_argument('--raw_file', help='the optional file full path of the WJH raw packets')
parser.add_argument('--agg_file', help='the optional file full path of the WJH aggregation info')
parser.add_argument('--enable_drop_reason_group',
                    choices=['all', 'l1', 'l2', 'router', 'tunnel', 'acl', 'buffer'],
                    help='enable drop reason group')
parser.add_argument('--disable_drop_reason_group',
                    choices=['all', 'l1', 'l2', 'router', 'tunnel', 'acl', 'buffer'],
                    help='disable drop reason group')
parser.add_argument('--severity', default='all',
                    choices=['all', 'notice', 'warning', 'error'],
                    help='drop reason severity for drop reason group disable/enable')
parser.add_argument('--bind_to_aggregation_channel',
                    choices=['all', 'l1', 'l2', 'router', 'tunnel', 'acl', 'buffer'],
                    help='bind drop reason group to aggregation channel')
parser.add_argument('--bind_to_cyclic_channel',
                    choices=['all', 'l1', 'l2', 'router', 'tunnel', 'acl', 'buffer'],
                    help='bind drop reason group to cyclic channel')
parser.add_argument('--bind_to_cyclic_and_aggregation_channel',
                    choices=['all', 'l1', 'l2', 'router', 'tunnel', 'acl', 'buffer'],
                    help='bind drop reason group to cyclic-and-aggregation channel')
parser.add_argument('--set_aggregation_channel_aggregation_read_mode',
                    choices=['read_clear', 'read'],
                    help='set the aggregation read mode of the aggregation channel')
parser.add_argument('--set_cyclic_and_aggregation_channel_aggregation_read_mode',
                    choices=['read_clear', 'read'],
                    help='set the aggregation read mode of the cyclic-and-aggregation channel')
parser.add_argument('--set_channel_timestamp_source',
                    choices=['linux', 'hw_clock'],
                    help='set the channel timestamp source')

cmd = pickle.dumps(sys.argv[1:])
args = parser.parse_args()

if args.enable:
    wjh_action_enable(cmd)
    if args.mode == 'agent':
        signal.signal(signal.SIGINT, handler)
        raw_input("Press any key to disable WJH example backend.")
        cmd = pickle.dumps(['--disable'])
        wjh_action_disable(cmd)
elif args.disable:
    wjh_action_disable(cmd)
elif args.save:
    wjh_action_save(cmd)
elif vars(args)['print']:
    wjh_action_print(cmd)
elif args.enable_drop_reason_group:
    wjh_action_enable_group(cmd)
elif args.disable_drop_reason_group:
    wjh_action_disable_group(cmd)
elif args.bind_to_aggregation_channel:
    wjh_action_bind_to_aggregation_channel(cmd)
elif args.bind_to_cyclic_channel:
    wjh_action_bind_to_cyclic_channel(cmd)
elif args.bind_to_cyclic_and_aggregation_channel:
    wjh_action_bind_to_cyclic_and_aggregation_channel(cmd)
elif args.set_aggregation_channel_aggregation_read_mode:
    wjh_action_set_aggregation_channel_aggregation_read_mode(cmd)
elif args.set_cyclic_and_aggregation_channel_aggregation_read_mode:
    wjh_action_set_cyclic_and_aggregation_channel_aggregation_read_mode(cmd)
elif args.set_channel_timestamp_source:
    wjh_action_set_channel_timestamp_source(cmd)
else:
    print("usage: wjh_example.py [-h|--help] [--enable] [--disable] [--print] [--save] [--enable_drop_reason_group] [--disable_drop_reason_group] [--severity] [--bind_to_aggregation_channel] [--bind_to_cyclic_channel] [--bind_to_cyclic_and_aggregation_channel]")
