#!/usr/bin/env python
'''
This file contains python based "What just happened" library backend for processing commands send by the WJH example frontend.
The backend is started by executing the wjh_example.py --enable commands. The WJH backend processes the following options from frontend
This is supported on Spectrum/Spectrum-2 devices.
    -h, --help            show this help message and exit
    --disable             disables the WJH example backend
    --enable              enables the WJH example backend
    --bandwidth BANDWIDTH
                          set pcie bandwidth percentage for WJH for enable
                          option
    --mode {agent,manual}
                          set WJH example mode for enable option
    --print               prints the WJH captured packets
    --save                saves the WJH captured packets
    --raw_file RAW_FILE   the optional file full path of the WJH raw packets
    --agg_file AGG_FILE   the optional file full path of the WJH aggregation
                          info
    --enable_drop_reason_group {all,l1,l2,router,tunnel,acl,buffer}
                          enable drop reason group
    --disable_drop_reason_group {all,l1,l2,router,tunnel,acl,buffer}
                          disable drop reason group
    --severity {all}      drop reason severity for drop reason group
                          disable/enable

'''
from __future__ import print_function
import sys
import argparse
import ctypes
from ctypes import *
import socket
import signal
import pickle
import struct
import os.path
from __builtin__ import True
from python_sdk_api.sx_api import *

######################################################
#    defines
######################################################
WJH_SOCKET_BUFFER_SIZE = 1024
WJH_MONITOR_SERVER_PORT = 20000
WJH_MAX_PKT_SIZE = 10000
WJH_PCAP_NAME = "/tmp/wjh_raw.pcap"
WJH_TXT_NAME = "/tmp/wjh_agg.txt"
WJH_SERVER_LISTEN_QUEUE_SIZE = 1

WJH_USER_CHANNEL_TAILDROP_E = 0
WJH_USER_CHANNEL_CYCLIC_E = 1
WJH_USER_CHANNEL_AGGREGATE_E = 2
WJH_USER_CHANNEL_CYCLIC_AND_AGGREGATE_E = 3

WJH_DROP_REASON_GROUP_BUFFER_E = 0
WJH_DROP_REASON_GROUP_ACL_E = 1
WJH_DROP_REASON_GROUP_L1_E = 2
WJH_DROP_REASON_GROUP_L2_E = 3
WJH_DROP_REASON_GROUP_ROUTER_E = 4
WJH_DROP_REASON_GROUP_TUNNEL_E = 5
WJH_DROP_REASON_GROUP_ALL_E = 6

WJH_USER_CHANNEL_MODE_PUSH_E = 0
WJH_USER_CHANNEL_MODE_PULL_E = 1

WJH_SEVERITY_ALL_E = 0
WJH_SEVERITY_NOTICE_E = 1
WJH_SEVERITY_WARNING_E = 2
WJH_SEVERITY_ERROR_E = 3

WJH_ACL_BINDING_POINT_INGRESS = 0
WJH_ACL_BINDING_POINT_EGRESS = 1
WJH_ACL_BINDING_POINT_RIF_INGRESS = 2
WJH_ACL_BINDING_POINT_RIF_EGRESS = 3

WJH_INGRESS_INFO_TYPE_LOGPORT = 0
WJH_INGRESS_INFO_TYPE_IF_INDEX = 1

WJH_AGGREGATION_READ_MODE_READ_CLEAR = 0
WJH_AGGREGATION_READ_MODE_READ = 1

WJH_AGGREGATION_KEY_MODE_STREAMING_E = 0
WJH_AGGREGATION_KEY_MODE_DETAILED_E = 1

WJH_USER_CHANNEL_TIMESTAMP_SOURCE_LINUX_E = 0
WJH_USER_CHANNEL_TIMESTAMP_SOURCE_HW_CLOCK_E = 1

RECYCLE_PORT = 0x10100
RECYCLE_PORT_PG_TC = 1
ING_POOL_ID = 0
EGR_POOL_ID = 11
ING_DESC_POOL_ID = 21
EGR_DESC_POOL_ID = 30

SX_COS_ING_PG_0_E = 0
SX_COS_ING_PG_1_E = 1
SX_COS_ING_PG_7_E = 7
SX_COS_EGR_TC_0_E = 0
SX_COS_EGR_TC_1_E = 1
SX_COS_EGR_TC_15_E = 15

port_def_pg_buffer = new_sx_cos_port_buffer_attr_t_arr(SX_COS_ING_PG_7_E + 1)
port_def_pg_desc_buffer = new_sx_cos_port_buffer_attr_t_arr(SX_COS_ING_PG_7_E + 1)
port_def_tc_buffer = new_sx_cos_port_buffer_attr_t_arr(SX_COS_EGR_TC_15_E + 1)
port_def_tc_desc_buffer = new_sx_cos_port_buffer_attr_t_arr(SX_COS_EGR_TC_15_E + 1)
port_def_ing_port_buffer = sx_cos_port_buffer_attr_t()
port_def_egr_port_buffer = sx_cos_port_buffer_attr_t()
port_def_ing_port_desc_buffer = sx_cos_port_buffer_attr_t()
port_def_egr_port_desc_buffer = sx_cos_port_buffer_attr_t()

port_def_pg_shared_buffer = new_sx_cos_port_shared_buffer_attr_t_arr(SX_COS_ING_PG_7_E + 1)
port_def_pg_desc_shared_buffer = new_sx_cos_port_shared_buffer_attr_t_arr(SX_COS_ING_PG_7_E + 1)
port_def_tc_shared_buffer = new_sx_cos_port_shared_buffer_attr_t_arr(SX_COS_EGR_TC_15_E + 1)
port_def_tc_desc_shared_buffer = new_sx_cos_port_shared_buffer_attr_t_arr(SX_COS_EGR_TC_15_E + 1)
port_def_ing_port_shared_buffer = sx_cos_port_shared_buffer_attr_t()
port_def_egr_port_shared_buffer = sx_cos_port_shared_buffer_attr_t()
port_def_ing_port_desc_shared_buffer = sx_cos_port_shared_buffer_attr_t()
port_def_egr_port_desc_shared_buffer = sx_cos_port_shared_buffer_attr_t()
def_prio_to_pg_mapping = {}
prio_to_tc_mapping = {}
PORT_PRIO_MAX = 15

channel_type_to_str = {
    WJH_USER_CHANNEL_TAILDROP_E: 'Taildrop',
    WJH_USER_CHANNEL_CYCLIC_E: 'Cyclic',
    WJH_USER_CHANNEL_AGGREGATE_E: 'Aggregate',
    WJH_USER_CHANNEL_CYCLIC_AND_AGGREGATE_E: 'Cyclic-and-Aggregate'
}

drop_reason_group_to_str = {
    WJH_DROP_REASON_GROUP_BUFFER_E: "buffer",
    WJH_DROP_REASON_GROUP_ACL_E: "acl",
    WJH_DROP_REASON_GROUP_L1_E: "l1",
    WJH_DROP_REASON_GROUP_L2_E: "l2",
    WJH_DROP_REASON_GROUP_ROUTER_E: "router",
    WJH_DROP_REASON_GROUP_TUNNEL_E: "tunnel",
    WJH_DROP_REASON_GROUP_ALL_E: "all"
}

drop_reason_group_str_to_enum = {
    "buffer": WJH_DROP_REASON_GROUP_BUFFER_E,
    "acl": WJH_DROP_REASON_GROUP_ACL_E,
    "l1": WJH_DROP_REASON_GROUP_L1_E,
    "l2": WJH_DROP_REASON_GROUP_L2_E,
    "router": WJH_DROP_REASON_GROUP_ROUTER_E,
    "tunnel": WJH_DROP_REASON_GROUP_TUNNEL_E,
    "all": WJH_DROP_REASON_GROUP_ALL_E
}

channel_mode_str_to_enum = {
    "agent": WJH_USER_CHANNEL_MODE_PUSH_E,
    "manual": WJH_USER_CHANNEL_MODE_PULL_E
}

severity_to_str = {
    WJH_SEVERITY_NOTICE_E: 'notice',
    WJH_SEVERITY_WARNING_E: 'warning',
    WJH_SEVERITY_ERROR_E: 'error',
    WJH_SEVERITY_ALL_E: 'all'
}

severity_str_to_enum = {
    "notice": WJH_SEVERITY_NOTICE_E,
    "warning": WJH_SEVERITY_WARNING_E,
    "error": WJH_SEVERITY_ERROR_E,
    "all": WJH_SEVERITY_ALL_E,
}

binding_point_to_str = {
    WJH_ACL_BINDING_POINT_INGRESS: 'Ingress port ACL',
    WJH_ACL_BINDING_POINT_EGRESS: 'Egress port ACL',
    WJH_ACL_BINDING_POINT_RIF_INGRESS: 'Ingress router ACL',
    WJH_ACL_BINDING_POINT_RIF_EGRESS: 'Egress router ACL'
}

agg_read_mode_str_to_enum = {
    "read_clear": WJH_AGGREGATION_READ_MODE_READ_CLEAR,
    "read": WJH_AGGREGATION_READ_MODE_READ
}

agg_key_mode_str_to_enum = {
    "streaming": WJH_AGGREGATION_KEY_MODE_STREAMING_E,
    "detailed": WJH_AGGREGATION_KEY_MODE_DETAILED_E
}

channel_timestamp_source_str_to_enum = {
    "linux": WJH_USER_CHANNEL_TIMESTAMP_SOURCE_LINUX_E,
    "hw_clock": WJH_USER_CHANNEL_TIMESTAMP_SOURCE_HW_CLOCK_E
}

drop_reason_group_to_channel = {}
WJH_MIRROR_CONG_INVALID = 0xFFFF
WJH_MIRROR_TCLASS_INVALID = 0x1F
WJH_MIRROR_LATENCY_INVALID = 0xFFFFFF

MELLANOX_WJH_HEADER = 'Mellanox WJH'

WJH_META_DATA_SOURCE_PORT = 1
WJH_META_DATA_DEST_PORT = 2
WJH_META_DATA_ORIG_PACKET_SIZE = 3
WJH_META_DATA_DROP_GROUP = 4
WJH_META_DATA_DROP_REASON = 5
WJH_META_DATA_MIN = 1
WJH_META_DATA_MAX = 5

PCAP_VERSION_MAJOR = 2
PCAP_VERSION_MINOR = 4
PCAP_MAGIC = 0xa1b2c3d4
PCAP_ERRBUF_SIZE = 256
PCAP_MAX_SNAPLEN = 65535
LINKTYPE_USER15 = 162

WjhDeactiveCb = CFUNCTYPE(None)

raw_channel = 0
aggregation_channel = 0
cyclic_and_aggregation_channel = 0
channel_mode = WJH_USER_CHANNEL_MODE_PUSH_E
channel_to_agg_read_mode = {}

enable_print = False
enable_save = False

pcap_fd = -1
txt_fd = -1
handle = -1  # SDK handle

pcap_user_fd = -1
txt_user_fd = -1

wjh_raw_file = WJH_PCAP_NAME
wjh_agg_file = WJH_TXT_NAME


class WjhInitParam(Structure):
    _fields_ = [("force", c_uint8), ("deactive_cb", WjhDeactiveCb), ("bandwidth_percent", c_uint8),
                ("log_cb", c_void_p), ("conf_xml_path", c_char_p), ("debug_fs_path", c_char_p), ("ingress_info_type", c_int),
                ("aggregation_key_mode", c_int)]


class WjhDropCounter(Structure):
    _fields_ = [("dropped", c_uint64), ("received", c_uint64)]


class WjhTimespec(Structure):
    _fields_ = [("tv_sec", c_long), ("tv_nsec", c_long)]


class WjhDropReason(Structure):
    _fields_ = [("id", c_uint32),
                ("reason", c_char_p),
                ("severity", c_int),
                ("description", c_char_p)]


class WjhDropBasicAttr(Structure):
    _fields_ = [("recirculation_port", c_uint32),
                ("truncate_size", c_uint16)]


class WjhDropAdvancedAttr(Structure):
    _fields_ = [("trap_probability", c_uint32),
                ("trap_id", c_uint32),
                ("ptp_disabled", c_uint32)]


class WjhBufferDropAttr(Structure):
    _fields_ = [("basic_attr", WjhDropBasicAttr),
                ("advanced_attr", WjhDropAdvancedAttr)]


class WjhDropReasonGroupAttrUnion(Union):
    _fields_ = [("buffer_drop", WjhBufferDropAttr)]


class WjhDropReasonGroupAttr(Structure):
    _fields_ = [("attr", WjhDropReasonGroupAttrUnion),
                ("max_aggregation_entries_cnt", c_uint32)]


class WjhUserChannelAttr(Structure):
    _fields_ = [("polling_interval", c_uint32),
                ("mode", c_int),
                ("aggregation_read_mode", c_int),
                ("timestamp_source", c_int)]


class WjhDropAggregateAttr(Structure):
    _fields_ = [("mode", c_int)]


class WjhL2DropRawInfo(Structure):
    _fields_ = [("packet", c_void_p),
                ("packet_size", c_uint32),
                ("ingress_port", c_uint32),
                ("drop_reason", WjhDropReason),
                ("timestamp", WjhTimespec),
                ("is_lag_member", c_uint8),
                ("ingress_lag", c_uint32)]


WjhL2DropRawCb = CFUNCTYPE(c_int, POINTER(WjhL2DropRawInfo), POINTER(c_uint32))


class WjhRouterDropRawInfo(Structure):
    _fields_ = [("packet", c_void_p),
                ("packet_size", c_uint32),
                ("ingress_port", c_uint32),
                ("drop_reason", WjhDropReason),
                ("timestamp", WjhTimespec),
                ("is_lag_member", c_uint8),
                ("ingress_lag", c_uint32)]


WjhRouterDropRawCb = CFUNCTYPE(c_int, POINTER(WjhRouterDropRawInfo), POINTER(c_uint32))


class WjhTunnelDropRawInfo(Structure):
    _fields_ = [("packet", c_void_p),
                ("packet_size", c_uint32),
                ("ingress_port", c_uint32),
                ("drop_reason", WjhDropReason),
                ("timestamp", WjhTimespec),
                ("is_lag_member", c_uint8),
                ("ingress_lag", c_uint32)]


WjhTunnelDropRawCb = CFUNCTYPE(c_int, POINTER(WjhTunnelDropRawInfo), POINTER(c_uint32))


class WjhL1DropRawInfo(Structure):
    _fields_ = [("ingress_port", c_uint32),
                ("timestamp", WjhTimespec),
                ("reason", WjhDropReason)]


WjhL1DropRawCb = CFUNCTYPE(c_int, POINTER(WjhL1DropRawInfo), POINTER(c_uint32))


class WjhL1DropAggregateKey(Structure):
    _fields_ = [("ingress_port", c_uint32)]


class WjhAggregateTimestamp(Structure):
    _fields_ = [("first_timestamp", WjhTimespec), ("last_timestamp", WjhTimespec)]


class WjhL1DropAggregateData(Structure):
    _fields_ = [("is_port_up", c_uint8),
                ("port_down_reason", c_char_p),
                ("description", c_char_p),
                ("state_change_count", c_uint64),
                ("symbol_error_count", c_uint64),
                ("crc_error_count", c_uint64),
                ("timestamp", WjhAggregateTimestamp)]


WjhL1DropAggregateCb = CFUNCTYPE(c_int, POINTER(WjhL1DropAggregateKey), POINTER(WjhL1DropAggregateData), POINTER(c_uint32), POINTER(WjhDropAggregateAttr))


class WjhAclDropRawInfo(Structure):
    _fields_ = [("packet", c_void_p),
                ("packet_size", c_uint32),
                ("ingress_port", c_uint32),
                ("binding_point", c_int),
                ("rule_id", c_uint64),
                ("acl_name", c_char_p),
                ("rule", c_char_p),
                ("timestamp", WjhTimespec),
                ("is_lag_member", c_uint8),
                ("ingress_lag", c_uint32),
                ("drop_reason", WjhDropReason)]


WjhAclDropRawCb = CFUNCTYPE(c_int, POINTER(WjhAclDropRawInfo), POINTER(c_uint32))


class WjhBufferDropRawInfo(Structure):
    _fields_ = [("packet", ctypes.c_void_p),
                ("packet_size", ctypes.c_uint32),
                ("ingress_port", ctypes.c_uint32),
                ("egress_port", ctypes.c_uint32),
                ("tc", ctypes.c_uint8),
                ("drop_reason", WjhDropReason),
                ("timestamp", WjhTimespec),
                ("is_lag_member", ctypes.c_uint8),
                ("ingress_lag", ctypes.c_uint32),
                ("egress_port_valid", ctypes.c_uint8),
                ("is_egress_lag_member", ctypes.c_uint8),
                ("egress_lag", ctypes.c_uint32),
                ("original_occupancy", ctypes.c_uint16),
                ("original_latency", ctypes.c_uint32)]


WjhBufferDropRawCb = CFUNCTYPE(c_int, POINTER(WjhBufferDropRawInfo), POINTER(c_uint32))


class WjhAggregateFiveTuples(Structure):
    _fields_ = [("sip", c_char_p),
                ("dip", c_char_p),
                ("proto", c_uint8),
                ("sport", c_uint32),
                ("dport", c_uint32)]


class WjhBufferDropAggregateKey(Structure):
    _fields_ = [("five_tuples", WjhAggregateFiveTuples),
                ("non_ip", c_uint8),
                ("ingress_port", c_uint32),
                ("is_lag_member", ctypes.c_uint8),
                ("ingress_lag", ctypes.c_uint32),
                ("vlan", ctypes.c_uint16),
                ("ether_type", ctypes.c_uint16),
                ("dmac", c_uint8 * 6),
                ("smac", c_uint8 * 6),
                ("reason", WjhDropReason)]


class WjhBufferDropAggregateData(Structure):
    _fields_ = [("timestamp", WjhAggregateTimestamp),
                ("count", c_uint64)]


class WjhAclDropAggregateKey(Structure):
    _fields_ = [("five_tuples", WjhAggregateFiveTuples),
                ("non_ip", c_uint8),
                ("ingress_port", c_uint32),
                ("is_lag_member", ctypes.c_uint8),
                ("ingress_lag", ctypes.c_uint32),
                ("vlan", ctypes.c_uint16),
                ("ether_type", ctypes.c_uint16),
                ("dmac", c_uint8 * 6),
                ("smac", c_uint8 * 6),
                ("rule_id", c_uint64),
                ("acl_name", c_char_p),
                ("rule", c_char_p),
                ("reason", WjhDropReason)]


class WjhAclDropAggregateData(Structure):
    _fields_ = [("timestamp", WjhAggregateTimestamp),
                ("count", c_uint64),
                ("flows_num", c_uint64)]


class WjhL2DropAggregateKey(Structure):
    _fields_ = [("five_tuples", WjhAggregateFiveTuples),
                ("non_ip", c_uint8),
                ("ingress_port", c_uint32),
                ("is_lag_member", ctypes.c_uint8),
                ("ingress_lag", ctypes.c_uint32),
                ("vlan", ctypes.c_uint16),
                ("ether_type", ctypes.c_uint16),
                ("dmac", c_uint8 * 6),
                ("smac", c_uint8 * 6),
                ("reason", WjhDropReason)]


class WjhL2DropAggregateData(Structure):
    _fields_ = [("timestamp", WjhAggregateTimestamp),
                ("count", c_uint64)]


class WjhRouterDropAggregateKey(Structure):
    _fields_ = [("five_tuples", WjhAggregateFiveTuples),
                ("non_ip", c_uint8),
                ("ingress_port", c_uint32),
                ("is_lag_member", ctypes.c_uint8),
                ("ingress_lag", ctypes.c_uint32),
                ("vlan", ctypes.c_uint16),
                ("ether_type", ctypes.c_uint16),
                ("dmac", c_uint8 * 6),
                ("smac", c_uint8 * 6),
                ("reason", WjhDropReason)]


class WjhRouterDropAggregateData(Structure):
    _fields_ = [("timestamp", WjhAggregateTimestamp),
                ("count", c_uint64)]


class WjhTunnelDropAggregateKey(Structure):
    _fields_ = [("five_tuples", WjhAggregateFiveTuples),
                ("non_ip", c_uint8),
                ("ingress_port", c_uint32),
                ("is_lag_member", ctypes.c_uint8),
                ("ingress_lag", ctypes.c_uint32),
                ("vlan", ctypes.c_uint16),
                ("ether_type", ctypes.c_uint16),
                ("dmac", c_uint8 * 6),
                ("smac", c_uint8 * 6),
                ("reason", WjhDropReason)]


class WjhTunnelDropAggregateData(Structure):
    _fields_ = [("timestamp", WjhAggregateTimestamp),
                ("count", c_uint64)]


WjhBufferDropAggregateCb = CFUNCTYPE(c_int, POINTER(WjhBufferDropAggregateKey), POINTER(WjhBufferDropAggregateData), POINTER(c_uint32), POINTER(WjhDropAggregateAttr))
WjhAclDropAggregateCb = CFUNCTYPE(c_int, POINTER(WjhAclDropAggregateKey), POINTER(WjhAclDropAggregateData), POINTER(c_uint32), POINTER(WjhDropAggregateAttr))
WjhL2DropAggregateCb = CFUNCTYPE(c_int, POINTER(WjhL2DropAggregateKey), POINTER(WjhL2DropAggregateData), POINTER(c_uint32), POINTER(WjhDropAggregateAttr))
WjhRouterDropAggregateCb = CFUNCTYPE(c_int, POINTER(WjhRouterDropAggregateKey), POINTER(WjhRouterDropAggregateData), POINTER(c_uint32), POINTER(WjhDropAggregateAttr))
WjhTunnelDropAggregateCb = CFUNCTYPE(c_int, POINTER(WjhTunnelDropAggregateKey), POINTER(WjhTunnelDropAggregateData), POINTER(c_uint32), POINTER(WjhDropAggregateAttr))


def wjh_deactive_cb():
    print("This WJH client is deactivated by another WJH client")
    sys.exit(0)


def prepare_pcap(fd):
    thiszone = 0
    sigfigs = 0
    pcap_hdr = struct.pack('IHHIIII', PCAP_MAGIC, PCAP_VERSION_MAJOR, PCAP_VERSION_MINOR, thiszone, sigfigs, PCAP_MAX_SNAPLEN, LINKTYPE_USER15)
    fd.write(pcap_hdr)


def open_new_files():
    global pcap_fd, txt_fd

    pcap_fd = open(WJH_PCAP_NAME, "w")
    txt_fd = open(WJH_TXT_NAME, "w")

    prepare_pcap(pcap_fd)


def save_packet_pcap(drop_reason_group, raw_info, packet):
    global pcap_fd, pcap_user_fd

    if pcap_user_fd > 0:
        fd = pcap_user_fd
    else:
        fd = pcap_fd

    src_port = "0x%X" % raw_info.ingress_port
    dest_port = 'N/A'
    packet_size = str(raw_info.packet_size)
    drop_group = drop_reason_group_to_str[drop_reason_group]
    if drop_reason_group == WJH_DROP_REASON_GROUP_ACL_E:
        drop_reason = binding_point_to_str[raw_info.binding_point]
    else:
        drop_reason = raw_info.drop_reason.reason

    src_port_buf = struct.pack('BB', WJH_META_DATA_SOURCE_PORT, len(src_port)) + src_port
    dest_port_buf = struct.pack('BB', WJH_META_DATA_DEST_PORT, len(dest_port)) + dest_port
    packet_size_buf = struct.pack('BB', WJH_META_DATA_ORIG_PACKET_SIZE, len(packet_size)) + packet_size
    drop_group_buf = struct.pack('BB', WJH_META_DATA_DROP_GROUP, len(drop_group)) + drop_group
    drop_reason_buf = struct.pack('BB', WJH_META_DATA_DROP_REASON, len(drop_reason)) + drop_reason

    meta_data_buff = src_port_buf + dest_port_buf + packet_size_buf + drop_group_buf + drop_reason_buf

    wjhhdr_buf = MELLANOX_WJH_HEADER

    meta_data_len = len(meta_data_buff)
    mellanox_header_len = len(MELLANOX_WJH_HEADER)
    packet_len = raw_info.packet_size

    hdr_len = packet_len + mellanox_header_len + meta_data_len
    hdr_caplen = packet_len + mellanox_header_len + meta_data_len
    hdr_tv_sec = raw_info.timestamp.tv_sec
    hdr_tv_usec = raw_info.timestamp.tv_nsec / 1000000
    pkthdr_buf = struct.pack('IIII', hdr_tv_sec, hdr_tv_usec, hdr_caplen, hdr_len)

    packet_buf = packet

    fd.write(pkthdr_buf)
    fd.write(wjhhdr_buf)
    fd.write(meta_data_buff)
    fd.write(packet_buf)

    fd.flush()

    return


def wjh_print_raw_meta(drop_reason_group, raw_info):
    print("%s drop reason group: ingress port [0x%x], size[%d], id[%d], reason[%s], description[%s], severity[%s], timestamp[%d.%d]."
          % (drop_reason_group_to_str[drop_reason_group],
             raw_info.ingress_port,
             raw_info.packet_size,
             raw_info.drop_reason.id,
             raw_info.drop_reason.reason,
             raw_info.drop_reason.description,
             severity_to_str[raw_info.drop_reason.severity],
             raw_info.timestamp.tv_sec,
             raw_info.timestamp.tv_nsec))

    if drop_reason_group is WJH_DROP_REASON_GROUP_ACL_E:
        print("binding point[%s], rule_id(0x%x), acl_name[%s], rule{%s}"
              % (binding_point_to_str[raw_info.binding_point],
                 raw_info.rule_id,
                 raw_info.acl_name,
                 raw_info.rule))

    if drop_reason_group is WJH_DROP_REASON_GROUP_BUFFER_E:
        if raw_info.egress_port_valid:
            print ("egress port [0x%x], " % raw_info.egress_port)
        if raw_info.original_occupancy != WJH_MIRROR_CONG_INVALID:
            print ("Tclass Buffer Occupancy [%u] KB, " % (raw_info.original_occupancy * 8))
        if raw_info.tc != WJH_MIRROR_TCLASS_INVALID:
            print ("Egress Tclass [%u], " % (raw_info.tc))
        if raw_info.original_latency != WJH_MIRROR_LATENCY_INVALID:
            print ("Latency [%lu] nsec" % (raw_info.original_latency * 32))

    return


def wjh_process_raw_packet(drop_reason_group, raw_info_list_p, raw_info_list_size_p):
    list_size = raw_info_list_size_p.contents.value
    if enable_print:
        print("")
    for i in range(list_size):
        packet_size = raw_info_list_p[i].packet_size

        if enable_print:
            wjh_print_raw_meta(drop_reason_group, raw_info_list_p[i])

            packet = list((packet_size * ctypes.c_ubyte).from_address(raw_info_list_p[i].packet))

            if packet_size > 0:
                for ii in range(0, packet_size):
                    if (0 == ii % 16) and (ii > 0):
                        print("")
                    print("%02x" % packet[ii], end=' ')
                print("")

        if enable_save:
            buf = (ctypes.c_char * packet_size)()
            ctypes.memmove(buf, raw_info_list_p[i].packet, packet_size)
            packet_str = ctypes.string_at(buf, packet_size)
            save_packet_pcap(drop_reason_group, raw_info_list_p[i], packet_str)

    return 0


def wjh_l2_drop_raw_cb(raw_info_list_p, raw_info_list_size_p):
    rc = wjh_process_raw_packet(WJH_DROP_REASON_GROUP_L2_E, raw_info_list_p, raw_info_list_size_p)
    return rc


def wjh_router_drop_raw_cb(raw_info_list_p, raw_info_list_size_p):
    rc = wjh_process_raw_packet(WJH_DROP_REASON_GROUP_ROUTER_E, raw_info_list_p, raw_info_list_size_p)
    return rc


def wjh_tunnel_drop_raw_cb(raw_info_list_p, raw_info_list_size_p):
    rc = wjh_process_raw_packet(WJH_DROP_REASON_GROUP_TUNNEL_E, raw_info_list_p, raw_info_list_size_p)
    return rc


def wjh_acl_drop_raw_cb(raw_info_list_p, raw_info_list_size_p):
    rc = wjh_process_raw_packet(WJH_DROP_REASON_GROUP_ACL_E, raw_info_list_p, raw_info_list_size_p)
    return rc


def wjh_buffer_drop_raw_cb(raw_info_list_p, raw_info_list_size_p):
    rc = wjh_process_raw_packet(WJH_DROP_REASON_GROUP_BUFFER_E, raw_info_list_p, raw_info_list_size_p)
    return rc


def wjh_l1_drop_raw_cb(raw_info_list_p, raw_info_list_size_p):
    global txt_fd, txt_user_fd
    fd = -1
    if enable_print:
        fd = sys.stdout
    if enable_save:
        if txt_user_fd > 0:
            fd = txt_user_fd
        else:
            fd = txt_fd

    list_size = raw_info_list_size_p.contents.value

    if fd >= 0:
        print("", file=fd)
    for i in range(list_size):
        if fd >= 0:
            print("port 0x%x, timestamp[%ld.%ld], reason_str[%s], reason_id[%u], description[%s], severity[%s]" %
                  (raw_info_list_p[i].ingress_port, raw_info_list_p[i].timestamp.tv_sec,
                   raw_info_list_p[i].timestamp.tv_nsec, raw_info_list_p[i].reason.reason,
                   raw_info_list_p[i].reason.id, raw_info_list_p[i].reason.description,
                   severity_to_str[raw_info_list_p[i].reason.severity]), file=fd)

    if enable_save:
        txt_fd.flush()

    return 0


def wjh_l1_drop_aggregate_cb(key_list_p, data_list_p, keys_cnt_p, attr_p):
    global txt_fd, txt_user_fd
    fd = -1
    if enable_print:
        fd = sys.stdout
    if enable_save:
        if txt_user_fd > 0:
            fd = txt_user_fd
        else:
            fd = txt_fd

    list_size = keys_cnt_p.contents.value

    if fd >= 0:
        print("", file=fd)
    for i in range(list_size):
        if fd >= 0:
            print("port 0x%x: is_port_up: %u, port_down_reason: %s, description: %s, state_change_count: %u" %
                  (key_list_p[i].ingress_port, data_list_p[i].is_port_up, data_list_p[i].port_down_reason,
                   data_list_p[i].description, data_list_p[i].state_change_count), file=fd)
            print("           symbol_error_count: %u, crc_error_count: %u" %
                  (data_list_p[i].symbol_error_count, data_list_p[i].crc_error_count), file=fd)
            print("           timestamp[%ld.%ld, %ld.%ld]" % (data_list_p[i].timestamp.first_timestamp.tv_sec,
                                                              data_list_p[i].timestamp.first_timestamp.tv_nsec, data_list_p[i].timestamp.last_timestamp.tv_sec,
                                                              data_list_p[i].timestamp.last_timestamp.tv_nsec), file=fd)
            print("           user channel mode: %d" % (attr_p[0].mode), file=fd)

    if enable_save:
        txt_fd.flush()

    return 0


def wjh_buffer_drop_aggregate_cb(key_list_p, data_list_p, keys_cnt_p, attr_p):
    global txt_fd, txt_user_fd
    fd = -1
    if enable_print:
        fd = sys.stdout
    if enable_save:
        if txt_user_fd > 0:
            fd = txt_user_fd
        else:
            fd = txt_fd

    list_size = keys_cnt_p.contents.value

    if fd >= 0:
        print("", file=fd)
    for i in range(list_size):
        if fd >= 0:
            print("sip [%s], dip [%s], proto [%u], sport [%u], dport [%u], non_ip [%u], reason [%s]" %
                  (key_list_p[i].five_tuples.sip, key_list_p[i].five_tuples.dip, key_list_p[i].five_tuples.proto,
                   key_list_p[i].five_tuples.sport, key_list_p[i].five_tuples.dport, key_list_p[i].non_ip, key_list_p[i].reason.reason), file=fd)
            print("ingress_port [0x%x] is_lag_member [%u] ingress_lag [0x%x] vlan [%u] ether_type [0x%x]" %
                  (key_list_p[i].ingress_port, key_list_p[i].is_lag_member, key_list_p[i].ingress_lag,
                   key_list_p[i].vlan, key_list_p[i].ether_type), file=fd)
            print("dmac: %02x:%02x:%02x:%02x:%02x:%02x" % (key_list_p[i].dmac[0], key_list_p[i].dmac[1], key_list_p[i].dmac[2],
                                                           key_list_p[i].dmac[3], key_list_p[i].dmac[4], key_list_p[i].dmac[5]), file=fd)
            print("smac: %02x:%02x:%02x:%02x:%02x:%02x" % (key_list_p[i].smac[0], key_list_p[i].smac[1], key_list_p[i].smac[2],
                                                           key_list_p[i].smac[3], key_list_p[i].smac[4], key_list_p[i].smac[5]), file=fd)
            print("           count [%u], timestamp [%ld.%ld %ld.%ld]" %
                  (data_list_p[i].count, data_list_p[i].timestamp.first_timestamp.tv_sec,
                   data_list_p[i].timestamp.first_timestamp.tv_nsec, data_list_p[i].timestamp.last_timestamp.tv_sec,
                   data_list_p[i].timestamp.last_timestamp.tv_nsec), file=fd)
            print("           user channel mode: %d" % (attr_p[0].mode), file=fd)

    if enable_save:
        txt_fd.flush()

    return 0


def wjh_acl_drop_aggregate_cb(key_list_p, data_list_p, keys_cnt_p, attr_p):
    global txt_fd, txt_user_fd
    fd = -1
    if enable_print:
        fd = sys.stdout
    if enable_save:
        if txt_user_fd > 0:
            fd = txt_user_fd
        else:
            fd = txt_fd

    list_size = keys_cnt_p.contents.value

    if fd >= 0:
        print("", file=fd)
    for i in range(list_size):
        if fd >= 0:
            print("sip [%s], dip [%s], proto [%u], sport [%u], dport [%u], non_ip [%u], reason [%s]" %
                  (key_list_p[i].five_tuples.sip, key_list_p[i].five_tuples.dip, key_list_p[i].five_tuples.proto,
                   key_list_p[i].five_tuples.sport, key_list_p[i].five_tuples.dport, key_list_p[i].non_ip, key_list_p[i].reason.reason), file=fd)
            print("ingress_port [0x%x] is_lag_member [%u] ingress_lag [0x%x] vlan [%u] ether_type [0x%x]" %
                  (key_list_p[i].ingress_port, key_list_p[i].is_lag_member, key_list_p[i].ingress_lag,
                   key_list_p[i].vlan, key_list_p[i].ether_type), file=fd)
            print("dmac: %02x:%02x:%02x:%02x:%02x:%02x" % (key_list_p[i].dmac[0], key_list_p[i].dmac[1], key_list_p[i].dmac[2],
                                                           key_list_p[i].dmac[3], key_list_p[i].dmac[4], key_list_p[i].dmac[5]), file=fd)
            print("smac: %02x:%02x:%02x:%02x:%02x:%02x" % (key_list_p[i].smac[0], key_list_p[i].smac[1], key_list_p[i].smac[2],
                                                           key_list_p[i].smac[3], key_list_p[i].smac[4], key_list_p[i].smac[5]), file=fd)
            print("rule_id [%u], acl_name [%s], rule [%s]" %
                  (key_list_p[i].rule_id, key_list_p[i].acl_name, key_list_p[i].rule), file=fd)
            print("           count [%u], timestamp [%ld.%ld %ld.%ld]" %
                  (data_list_p[i].count, data_list_p[i].timestamp.first_timestamp.tv_sec,
                   data_list_p[i].timestamp.first_timestamp.tv_nsec, data_list_p[i].timestamp.last_timestamp.tv_sec,
                   data_list_p[i].timestamp.last_timestamp.tv_nsec), file=fd)
            print("           user channel mode: %d" % (attr_p[0].mode), file=fd)

    if enable_save:
        txt_fd.flush()

    return 0


def wjh_l2_drop_aggregate_cb(key_list_p, data_list_p, keys_cnt_p, attr_p):
    global txt_fd, txt_user_fd
    fd = -1
    if enable_print:
        fd = sys.stdout
    if enable_save:
        if txt_user_fd > 0:
            fd = txt_user_fd
        else:
            fd = txt_fd

    list_size = keys_cnt_p.contents.value

    if fd >= 0:
        print("", file=fd)
    for i in range(list_size):
        if fd >= 0:
            print("sip [%s], dip [%s], proto [%u], sport [%u], dport [%u], non_ip [%u], reason [%s]" %
                  (key_list_p[i].five_tuples.sip, key_list_p[i].five_tuples.dip, key_list_p[i].five_tuples.proto,
                   key_list_p[i].five_tuples.sport, key_list_p[i].five_tuples.dport, key_list_p[i].non_ip, key_list_p[i].reason.reason), file=fd)
            print("ingress_port [0x%x] is_lag_member [%u] ingress_lag [0x%x] vlan [%u] ether_type [0x%x]" %
                  (key_list_p[i].ingress_port, key_list_p[i].is_lag_member, key_list_p[i].ingress_lag,
                   key_list_p[i].vlan, key_list_p[i].ether_type), file=fd)
            print("dmac: %02x:%02x:%02x:%02x:%02x:%02x" % (key_list_p[i].dmac[0], key_list_p[i].dmac[1], key_list_p[i].dmac[2],
                                                           key_list_p[i].dmac[3], key_list_p[i].dmac[4], key_list_p[i].dmac[5]), file=fd)
            print("smac: %02x:%02x:%02x:%02x:%02x:%02x" % (key_list_p[i].smac[0], key_list_p[i].smac[1], key_list_p[i].smac[2],
                                                           key_list_p[i].smac[3], key_list_p[i].smac[4], key_list_p[i].smac[5]), file=fd)
            print("           count [%u], timestamp [%ld.%ld %ld.%ld]" %
                  (data_list_p[i].count, data_list_p[i].timestamp.first_timestamp.tv_sec,
                   data_list_p[i].timestamp.first_timestamp.tv_nsec, data_list_p[i].timestamp.last_timestamp.tv_sec,
                   data_list_p[i].timestamp.last_timestamp.tv_nsec), file=fd)
            print("           user channel mode: %d" % (attr_p[0].mode), file=fd)

    if enable_save:
        txt_fd.flush()

    return 0


def wjh_router_drop_aggregate_cb(key_list_p, data_list_p, keys_cnt_p, attr_p):
    global txt_fd, txt_user_fd
    fd = -1
    if enable_print:
        fd = sys.stdout
    if enable_save:
        if txt_user_fd > 0:
            fd = txt_user_fd
        else:
            fd = txt_fd

    list_size = keys_cnt_p.contents.value

    if fd >= 0:
        print("", file=fd)
    for i in range(list_size):
        if fd >= 0:
            print("sip [%s], dip [%s], proto [%u], sport [%u], dport [%u], non_ip [%u], reason [%s]" %
                  (key_list_p[i].five_tuples.sip, key_list_p[i].five_tuples.dip, key_list_p[i].five_tuples.proto,
                   key_list_p[i].five_tuples.sport, key_list_p[i].five_tuples.dport, key_list_p[i].non_ip, key_list_p[i].reason.reason), file=fd)
            print("ingress_port [0x%x] is_lag_member [%u] ingress_lag [0x%x] vlan [%u] ether_type [0x%x]" %
                  (key_list_p[i].ingress_port, key_list_p[i].is_lag_member, key_list_p[i].ingress_lag,
                   key_list_p[i].vlan, key_list_p[i].ether_type), file=fd)
            print("dmac: %02x:%02x:%02x:%02x:%02x:%02x" % (key_list_p[i].dmac[0], key_list_p[i].dmac[1], key_list_p[i].dmac[2],
                                                           key_list_p[i].dmac[3], key_list_p[i].dmac[4], key_list_p[i].dmac[5]), file=fd)
            print("smac: %02x:%02x:%02x:%02x:%02x:%02x" % (key_list_p[i].smac[0], key_list_p[i].smac[1], key_list_p[i].smac[2],
                                                           key_list_p[i].smac[3], key_list_p[i].smac[4], key_list_p[i].smac[5]), file=fd)
            print("           count [%u], timestamp [%ld.%ld %ld.%ld]" %
                  (data_list_p[i].count, data_list_p[i].timestamp.first_timestamp.tv_sec,
                   data_list_p[i].timestamp.first_timestamp.tv_nsec, data_list_p[i].timestamp.last_timestamp.tv_sec,
                   data_list_p[i].timestamp.last_timestamp.tv_nsec), file=fd)
            print("           user channel mode: %d" % (attr_p[0].mode), file=fd)

    if enable_save:
        txt_fd.flush()

    return 0


def wjh_tunnel_drop_aggregate_cb(key_list_p, data_list_p, keys_cnt_p, attr_p):
    global txt_fd, txt_user_fd
    fd = -1
    if enable_print:
        fd = sys.stdout
    if enable_save:
        if txt_user_fd > 0:
            fd = txt_user_fd
        else:
            fd = txt_fd

    list_size = keys_cnt_p.contents.value

    if fd >= 0:
        print("", file=fd)
    for i in range(list_size):
        if fd >= 0:
            print("sip [%s], dip [%s], proto [%u], sport [%u], dport [%u], non_ip [%u], reason [%s]" %
                  (key_list_p[i].five_tuples.sip, key_list_p[i].five_tuples.dip, key_list_p[i].five_tuples.proto,
                   key_list_p[i].five_tuples.sport, key_list_p[i].five_tuples.dport, key_list_p[i].non_ip, key_list_p[i].reason.reason), file=fd)
            print("ingress_port [0x%x] is_lag_member [%u] ingress_lag [0x%x] vlan [%u] ether_type [0x%x]" %
                  (key_list_p[i].ingress_port, key_list_p[i].is_lag_member, key_list_p[i].ingress_lag,
                   key_list_p[i].vlan, key_list_p[i].ether_type), file=fd)
            print("dmac: %02x:%02x:%02x:%02x:%02x:%02x" % (key_list_p[i].dmac[0], key_list_p[i].dmac[1], key_list_p[i].dmac[2],
                                                           key_list_p[i].dmac[3], key_list_p[i].dmac[4], key_list_p[i].dmac[5]), file=fd)
            print("smac: %02x:%02x:%02x:%02x:%02x:%02x" % (key_list_p[i].smac[0], key_list_p[i].smac[1], key_list_p[i].smac[2],
                                                           key_list_p[i].smac[3], key_list_p[i].smac[4], key_list_p[i].smac[5]), file=fd)
            print("           count [%u], timestamp [%ld.%ld %ld.%ld]" %
                  (data_list_p[i].count, data_list_p[i].timestamp.first_timestamp.tv_sec,
                   data_list_p[i].timestamp.first_timestamp.tv_nsec, data_list_p[i].timestamp.last_timestamp.tv_sec,
                   data_list_p[i].timestamp.last_timestamp.tv_nsec), file=fd)
            print("           user channel mode: %d" % (attr_p[0].mode), file=fd)

    if enable_save:
        txt_fd.flush()

    return 0


class RawCallback(Union):
    _fields_ = [("buffer", WjhBufferDropRawCb),
                ("acl", WjhAclDropRawCb),
                ("router", WjhRouterDropRawCb),
                ("L2", WjhL2DropRawCb),
                ("L1", WjhL1DropRawCb),
                ("tunnel", WjhTunnelDropRawCb)]


class AggregateCallback(Union):
    _fields_ = [("buffer", WjhBufferDropAggregateCb),
                ("acl", WjhAclDropAggregateCb),
                ("router", WjhRouterDropAggregateCb),
                ("L2", WjhL2DropAggregateCb),
                ("L1", WjhL1DropAggregateCb),
                ("tunnel", WjhTunnelDropAggregateCb)]


class WjhDropCallbacks(Structure):
    _fields_ = [("drop_reason_group", c_int),
                ("raw_cb", RawCallback),
                ("aggregate_cb", AggregateCallback)]


def create_user_channel(channel_type):
    user_channel = c_uint32()
    rc = wjh_lib.wjh_user_channel_create(channel_type, byref(user_channel))

    return user_channel.value


def get_counter(user_channel_id):
    drop_counter = WjhDropCounter()

    rc = wjh_lib.wjh_counter_get(user_channel_id, byref(drop_counter), 0)

    print("channel[%d]: drop_counter [%d], recieve_counter [%d]" % (user_channel_id, drop_counter.dropped, drop_counter.received))

    return drop_counter.dropped, drop_counter.received


def get_global_counter():
    drop_counter = WjhDropCounter()

    rc = wjh_lib.wjh_global_counter_get(byref(drop_counter), 0)

    print("global: drop_counter [%d], recieve_counter [%d]" % (drop_counter.dropped, drop_counter.received))

    return drop_counter.dropped, drop_counter.received


def configure_drop_reason_group(drop_reason_group):
    channel_id = raw_channel
    drop_reason_group_to_channel[drop_reason_group] = channel_id
    rc = wjh_lib.wjh_drop_reason_group_bind(drop_reason_group, channel_id)

    rc = wjh_lib.wjh_drop_reason_group_enable(drop_reason_group, WJH_SEVERITY_ALL_E)


def deconfigure_drop_reason_group(drop_reason_group):
    rc = wjh_lib.wjh_drop_reason_group_disable(drop_reason_group, WJH_SEVERITY_ALL_E)

    rc = wjh_lib.wjh_drop_reason_group_unbind(drop_reason_group)

    rc = wjh_lib.wjh_drop_reason_group_deinit(drop_reason_group)

# configure loopback mode for recirculation port


def configure_recirculation_port_loopback_mode(port):
    global handle
    rc = sx_api_port_state_set(handle, port, SX_PORT_ADMIN_STATUS_DOWN)
    assert rc == 0, "Failed to bring down the port"
    sx_api_port_phys_loopback_set(handle, port, SX_PORT_PHYS_LOOPBACK_ENABLE_INTERNAL)
    assert rc == 0, "Failed to put the port in loopback mode"
    sx_api_port_state_set(handle, port, SX_PORT_ADMIN_STATUS_UP)
    assert rc == 0, "Failed to bring up the port"


# disable loopback mode for recirculation port
def deconfigure_recirculation_port_loopback_mode(port):
    global handle
    rc = sx_api_port_state_set(handle, port, SX_PORT_ADMIN_STATUS_DOWN)
    assert rc == 0, "Failed to bring down the port"
    sx_api_port_phys_loopback_set(handle, port, SX_PORT_PHYS_LOOPBACK_DISABLE)
    assert rc == 0, "Failed to put the port in loopback mode"
    sx_api_port_state_set(handle, port, SX_PORT_ADMIN_STATUS_UP)
    assert rc == 0, "Failed to bring up the port"

# configure recirculation port's loopback, buffer and SP mapping


def configure_recirculation_port(analyzer_port):
    configure_recirculation_port_loopback_mode(analyzer_port)
    configure_recirculation_port_pg_tc_map(analyzer_port)
    configure_recirculation_port_buffer(analyzer_port)


# revert recirculation port's loopback, buffer and SP mapping
def deconfigure_recirculation_port(analyzer_port):
    deconfigure_recirculation_port_loopback_mode(analyzer_port)
    deconfigure_recirculation_port_pg_tc_map(analyzer_port)
    deconfigure_recirculation_port_buffer(analyzer_port)


# configure Switch Priority to PG and TC mapping
def configure_recirculation_port_pg_tc_map(analyzer_port):

    global def_prio_to_pg_mapping
    port = analyzer_port
    def_prio_to_pg_mapping = cos_port_pg_prio_buff_map_get(handle, port)

    new_prio_to_pg_mapping = {}
    for sp in range(0, PORT_PRIO_MAX):
        new_prio_to_pg_mapping[sp] = RECYCLE_PORT_PG_TC

    # set the new priority to PG mapping
    cos_port_prio_buff_map_to_pg(handle, port, prio_to_buff_dict=new_prio_to_pg_mapping)

    new_prio_to_tc_mapping = {}
    for sp in range(0, PORT_PRIO_MAX):
        new_prio_to_tc_mapping[sp] = RECYCLE_PORT_PG_TC
        # get the existing TC mapping
        prio_to_tc_mapping[sp] = cos_port_tc_prio_map_get(handle, port, sp)
        # set the new SP to TC mapping
        cos_port_prio_buff_map_to_tc(handle, port, sp, new_prio_to_tc_mapping[sp])

# Revert Switch Priority to PG and TC  mapping


def deconfigure_recirculation_port_pg_tc_map(analyzer_port):
    global def_prio_to_pg_mapping
    global prio_to_tc_mapping
    port = analyzer_port
    # revert the priority to PG mapping
    cos_port_prio_buff_map_to_pg(handle, port, prio_to_buff_dict=def_prio_to_pg_mapping)

    for sp in range(0, PORT_PRIO_MAX):
        # set the new SP to TC mapping
        cos_port_prio_buff_map_to_tc(handle, port, sp, prio_to_tc_mapping[sp])

# configure recirculation port buffer settings.


def configure_recirculation_port_buffer(analyzer_port):
    global port_def_pg_buffer, port_def_pg_desc_buffer, port_def_tc_buffer, port_def_tc_desc_buffer
    global port_def_ing_port_buffer, port_def_egr_port_buffer, port_def_ing_port_desc_buffer, port_def_egr_port_desc_buffer
    global port_def_pg_shared_buffer, port_def_pg_desc_shared_buffer, port_def_tc_shared_buffer, port_def_tc_desc_shared_buffer
    global port_def_ing_port_shared_buffer, port_def_egr_port_shared_buffer, port_def_ing_port_desc_shared_buffer, port_def_egr_port_desc_shared_buffer
    global def_prio_to_pg_mapping, prio_to_tc_mapping, handle

    in_pool_id = ING_POOL_ID
    eg_pool_id = EGR_POOL_ID
    in_desc_pool_id = ING_DESC_POOL_ID
    eg_desc_pool_id = EGR_DESC_POOL_ID
    buff_mode = SX_COS_BUFFER_MAX_MODE_DYNAMIC_E
    alpha0 = SX_COS_PORT_BUFF_ALPHA_0_E
    port = analyzer_port
    cmd = SX_ACCESS_CMD_SET

    data_res_size = ((128 * 1024) / 96)
    desc_res_size = 128

    port_buffer_attr_item = sx_cos_port_buffer_attr_t()
    port_shared_buffer_attr_item = sx_cos_port_shared_buffer_attr_t()

    # Save current configuration of ingress/egress port and desc reserved/shared buffers.
    # set the new configuration for analyzer port
    buff_type = SX_COS_INGRESS_PORT_ATTR_E
    port_buffer_attr_item = get_reserved_buffer_attributes(handle, port, buff_type, in_pool_id)
    port_def_ing_port_buffer = port_buffer_attr_item
    port_shared_buffer_attr_item = get_shared_buffer_attributes(handle, port, buff_type, in_pool_id)
    port_def_ing_port_shared_buffer = port_shared_buffer_attr_item
    modify_port_reserved_buffer(handle, cmd, port, buff_type, in_pool_id, size=data_res_size)
    modify_port_shared_buffer(handle, cmd, port, buff_type, in_pool_id, mode=buff_mode, alpha=alpha0)

    buff_type = SX_COS_EGRESS_PORT_ATTR_E
    port_buffer_attr_item = get_reserved_buffer_attributes(handle, port, buff_type, eg_pool_id)
    port_def_egr_port_buffer = port_buffer_attr_item
    port_shared_buffer_attr_item = get_shared_buffer_attributes(handle, port, buff_type, eg_pool_id)
    port_def_egr_port_shared_buffer = port_shared_buffer_attr_item
    modify_port_reserved_buffer(handle, cmd, port, buff_type, eg_pool_id, size=data_res_size)
    modify_port_shared_buffer(handle, cmd, port, buff_type, eg_pool_id, mode=buff_mode, alpha=alpha0)

    buff_type = SX_COS_PORT_BUFF_ATTR_RESERVED1_E
    port_buffer_attr_item = get_reserved_buffer_attributes(handle, port, buff_type, in_desc_pool_id)
    port_def_ing_port_desc_buffer = port_buffer_attr_item
    port_shared_buffer_attr_item = get_shared_buffer_attributes(handle, port, buff_type, in_desc_pool_id)
    port_def_ing_port_desc_shared_buffer = port_shared_buffer_attr_item
    modify_port_reserved_buffer(handle, cmd, port, buff_type, in_desc_pool_id, size=desc_res_size)
    modify_port_shared_buffer(handle, cmd, port, buff_type, in_desc_pool_id, mode=buff_mode, alpha=alpha0)

    buff_type = SX_COS_PORT_BUFF_ATTR_RESERVED3_E
    port_buffer_attr_item = get_reserved_buffer_attributes(handle, port, buff_type, eg_desc_pool_id)
    port_def_egr_port_desc_buffer = port_buffer_attr_item
    port_shared_buffer_attr_item = get_shared_buffer_attributes(handle, port, buff_type, eg_desc_pool_id)
    port_def_egr_port_desc_shared_buffer = port_shared_buffer_attr_item
    modify_port_reserved_buffer(handle, cmd, port, buff_type, eg_desc_pool_id, size=desc_res_size)
    modify_port_shared_buffer(handle, cmd, port, buff_type, eg_desc_pool_id, mode=buff_mode, alpha=alpha0)

    # Save current configuration of TC/PG and desc reserved/shared buffers
    # set the new configuration for analyzer port
    for tc_pg in range(0, 8):
        if tc_pg == RECYCLE_PORT_PG_TC:
            data_res_size = 128 * 1024 / 96
            desc_res_size = 128
        else:
            data_res_size = 0
            desc_res_size = 0

        buff_type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E
        port_buffer_attr_item = get_reserved_buffer_attributes(handle, port, buff_type, tc_pg)
        sx_cos_port_buffer_attr_t_arr_setitem(port_def_pg_buffer, tc_pg, port_buffer_attr_item)
        port_shared_buffer_attr_item = get_shared_buffer_attributes(handle, port, buff_type, tc_pg)
        sx_cos_port_shared_buffer_attr_t_arr_setitem(port_def_pg_shared_buffer, tc_pg, port_shared_buffer_attr_item)
        modify_port_reserved_buffer(handle, cmd, port, buff_type, tc_pg, size=data_res_size)
        modify_port_shared_buffer(handle, cmd, port, buff_type, tc_pg, pool_id=in_pool_id, mode=buff_mode, alpha=alpha0)

        buff_type = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E
        port_buffer_attr_item = get_reserved_buffer_attributes(handle, port, buff_type, tc_pg)
        sx_cos_port_buffer_attr_t_arr_setitem(port_def_tc_buffer, tc_pg, port_buffer_attr_item)
        port_shared_buffer_attr_item = get_shared_buffer_attributes(handle, port, buff_type, tc_pg)
        sx_cos_port_shared_buffer_attr_t_arr_setitem(port_def_tc_shared_buffer, tc_pg, port_shared_buffer_attr_item)
        modify_port_reserved_buffer(handle, cmd, port, buff_type, tc_pg, size=data_res_size)
        modify_port_shared_buffer(handle, cmd, port, buff_type, tc_pg, pool_id=eg_pool_id, mode=buff_mode, alpha=alpha0)

        buff_type = SX_COS_PORT_BUFF_ATTR_RESERVED2_E
        port_buffer_attr_item = get_reserved_buffer_attributes(handle, port, buff_type, tc_pg)
        sx_cos_port_buffer_attr_t_arr_setitem(port_def_pg_desc_buffer, tc_pg, port_buffer_attr_item)
        port_shared_buffer_attr_item = get_shared_buffer_attributes(handle, port, buff_type, tc_pg)
        sx_cos_port_shared_buffer_attr_t_arr_setitem(port_def_pg_desc_shared_buffer, tc_pg, port_shared_buffer_attr_item)
        modify_port_reserved_buffer(handle, cmd, port, buff_type, tc_pg, size=desc_res_size)
        modify_port_shared_buffer(handle, cmd, port, buff_type, tc_pg, pool_id=in_desc_pool_id, mode=buff_mode, alpha=alpha0)

        buff_type = SX_COS_PORT_BUFF_ATTR_RESERVED4_E
        port_buffer_attr_item = get_reserved_buffer_attributes(handle, port, buff_type, tc_pg)
        sx_cos_port_buffer_attr_t_arr_setitem(port_def_tc_desc_buffer, tc_pg, port_buffer_attr_item)
        port_shared_buffer_attr_item = get_shared_buffer_attributes(handle, port, buff_type, tc_pg)
        sx_cos_port_shared_buffer_attr_t_arr_setitem(port_def_tc_desc_shared_buffer, tc_pg, port_shared_buffer_attr_item)
        modify_port_reserved_buffer(handle, cmd, port, buff_type, tc_pg, size=desc_res_size)
        modify_port_shared_buffer(handle, cmd, port, buff_type, tc_pg, pool_id=eg_desc_pool_id, mode=buff_mode, alpha=alpha0)


# revert recirculation port buffer configuration.
def deconfigure_recirculation_port_buffer(analyzer_port):

    global port_def_pg_buffer, port_def_pg_desc_buffer, port_def_tc_buffer, port_def_tc_desc_buffer
    global port_def_ing_port_buffer, port_def_egr_port_buffer, port_def_ing_port_desc_buffer, port_def_egr_port_desc_buffer
    global port_def_pg_shared_buffer, port_def_pg_desc_shared_buffer, port_def_tc_shared_buffer, port_def_tc_desc_shared_buffer
    global port_def_ing_port_shared_buffer, port_def_egr_port_shared_buffer, port_def_ing_port_desc_shared_buffer, port_def_egr_port_desc_shared_buffer
    global def_prio_to_pg_mapping, prio_to_tc_mapping

    in_pool_id = ING_POOL_ID
    eg_pool_id = EGR_POOL_ID
    in_desc_pool_id = ING_DESC_POOL_ID
    eg_desc_pool_id = EGR_DESC_POOL_ID
    port = analyzer_port
    cmd = SX_ACCESS_CMD_SET

    port_buffer_attr_item = sx_cos_port_buffer_attr_t()
    port_shared_buffer_attr_item = sx_cos_port_shared_buffer_attr_t()
    # Revert the ingress/egress port and desc reserved and shared buffer configuration.
    buff_type = SX_COS_INGRESS_PORT_ATTR_E
    size = port_def_ing_port_buffer.attr.ingress_port_buff_attr.size
    modify_port_reserved_buffer(handle, cmd, port, buff_type, in_pool_id, size)
    buff_mode = port_def_ing_port_shared_buffer.attr.ingress_port_shared_buff_attr.max.mode
    alpha_val = port_def_ing_port_shared_buffer.attr.ingress_port_shared_buff_attr.max.max.alpha
    modify_port_shared_buffer(handle, cmd, port, buff_type, in_pool_id, mode=buff_mode, alpha=alpha_val)

    buff_type = SX_COS_EGRESS_PORT_ATTR_E
    size = port_def_egr_port_buffer.attr.egress_port_buff_attr.size
    modify_port_reserved_buffer(handle, cmd, port, buff_type, eg_pool_id, size)
    buff_mode = port_def_egr_port_shared_buffer.attr.egress_port_shared_buff_attr.max.mode
    alpha_val = port_def_egr_port_shared_buffer.attr.egress_port_shared_buff_attr.max.max.alpha
    modify_port_shared_buffer(handle, cmd, port, buff_type, eg_pool_id, mode=buff_mode, alpha=alpha_val)

    buff_type = SX_COS_PORT_BUFF_ATTR_RESERVED1_E
    size = port_def_ing_port_desc_buffer.attr.ingress_port_buff_attr.size
    modify_port_reserved_buffer(handle, cmd, port, buff_type, in_desc_pool_id, size)
    buff_mode = port_def_ing_port_desc_shared_buffer.attr.ingress_port_shared_buff_attr.max.mode
    alpha_val = port_def_ing_port_desc_shared_buffer.attr.ingress_port_shared_buff_attr.max.max.alpha
    modify_port_shared_buffer(handle, cmd, port, buff_type, in_desc_pool_id, mode=buff_mode, alpha=alpha_val)

    buff_type = SX_COS_PORT_BUFF_ATTR_RESERVED3_E
    size = port_def_egr_port_desc_buffer.attr.egress_port_buff_attr.size
    modify_port_reserved_buffer(handle, cmd, port, buff_type, eg_desc_pool_id, size)
    buff_mode = port_def_egr_port_desc_shared_buffer.attr.egress_port_shared_buff_attr.max.mode
    alpha_val = port_def_egr_port_desc_shared_buffer.attr.egress_port_shared_buff_attr.max.max.alpha
    modify_port_shared_buffer(handle, cmd, port, buff_type, eg_desc_pool_id, mode=buff_mode, alpha=alpha_val)

    # Revert configuration of shared and resreved buffers and desc buffers for tc/pg
    for tc_pg in range(0, 8):
        buff_type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E
        port_buffer_attr_item = sx_cos_port_buffer_attr_t_arr_getitem(port_def_pg_buffer, tc_pg)
        size = port_buffer_attr_item.attr.ingress_port_pg_buff_attr.size
        is_lossy = port_buffer_attr_item.attr.ingress_port_pg_buff_attr.is_lossy
        xon = port_buffer_attr_item.attr.ingress_port_pg_buff_attr.xon
        xoff = port_buffer_attr_item.attr.ingress_port_pg_buff_attr.xoff
        latency_override = port_buffer_attr_item.attr.ingress_port_pg_buff_attr.pipeline_latency.override_default
        latency_size = port_buffer_attr_item.attr.ingress_port_pg_buff_attr.pipeline_latency.size
        modify_port_reserved_buffer(handle, cmd, port, buff_type, tc_pg, size, is_lossy=is_lossy, xon=xon,
                                    xoff=xoff, pipeline_latency_override=latency_override, pipeline_latency_size=latency_size)
        port_shared_buffer_attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_def_pg_shared_buffer, tc_pg)
        buff_mode = port_shared_buffer_attr_item.attr.ingress_port_pg_shared_buff_attr.max.mode
        alpha_val = port_shared_buffer_attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.alpha
        modify_port_shared_buffer(handle, cmd, port, buff_type, tc_pg, pool_id=in_pool_id, mode=buff_mode, alpha=alpha_val)

        buff_type = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E
        port_buffer_attr_item = sx_cos_port_buffer_attr_t_arr_getitem(port_def_tc_buffer, tc_pg)
        size = port_buffer_attr_item.attr.egress_port_tc_buff_attr.size
        modify_port_reserved_buffer(handle, cmd, port, buff_type, tc_pg, size)
        port_shared_buffer_attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_def_tc_shared_buffer, tc_pg)
        buff_mode = port_shared_buffer_attr_item.attr.egress_port_tc_shared_buff_attr.max.mode
        alpha_val = port_shared_buffer_attr_item.attr.egress_port_tc_shared_buff_attr.max.max.alpha
        modify_port_shared_buffer(handle, cmd, port, buff_type, tc_pg, pool_id=eg_pool_id, mode=buff_mode, alpha=alpha_val)

        buff_type = SX_COS_PORT_BUFF_ATTR_RESERVED2_E
        port_buffer_attr_item = sx_cos_port_buffer_attr_t_arr_getitem(port_def_pg_desc_buffer, tc_pg)
        size = port_buffer_attr_item.attr.ingress_port_pg_buff_attr.size
        modify_port_reserved_buffer(handle, cmd, port, buff_type, tc_pg, size)
        port_shared_buffer_attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_def_pg_desc_shared_buffer, tc_pg)
        buff_mode = port_shared_buffer_attr_item.attr.ingress_port_pg_shared_buff_attr.max.mode
        alpha_val = port_shared_buffer_attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.alpha
        modify_port_shared_buffer(handle, cmd, port, buff_type, tc_pg, pool_id=in_desc_pool_id, mode=buff_mode, alpha=alpha_val)

        buff_type = SX_COS_PORT_BUFF_ATTR_RESERVED4_E
        port_buffer_attr_item = sx_cos_port_buffer_attr_t_arr_getitem(port_def_tc_desc_buffer, tc_pg)
        size = port_buffer_attr_item.attr.egress_port_tc_buff_attr.size
        modify_port_reserved_buffer(handle, cmd, port, buff_type, tc_pg, size)
        port_shared_buffer_attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_def_tc_desc_shared_buffer, tc_pg)
        buff_mode = port_shared_buffer_attr_item.attr.egress_port_tc_shared_buff_attr.max.mode
        alpha_val = port_shared_buffer_attr_item.attr.egress_port_tc_shared_buff_attr.max.max.alpha
        modify_port_shared_buffer(handle, cmd, port, buff_type, tc_pg, pool_id=eg_desc_pool_id, mode=buff_mode, alpha=alpha_val)


# API to get SP to PG mapping
def cos_port_pg_prio_buff_map_get(handle, log_port):
    prio_to_buff_get_p = new_sx_cos_port_prio_buff_t_p()
    try:
        rc = sx_api_cos_port_prio_buff_map_get(handle, log_port, prio_to_buff_get_p)
        assert rc == 0, "Failed to get pg, switch priority mapping"
        return {prio: sx_cos_port_buff_t_arr_getitem(prio_to_buff_get_p.prio_to_buff, prio) for prio in
                xrange(PORT_PRIO_MAX)}
    finally:
        delete_sx_cos_port_prio_buff_t_p(prio_to_buff_get_p)

# API to set SP to PG mapping


def cos_port_prio_buff_map_to_pg(handle, port, prio_id=0, pg_buffer_id=0, prio_to_buff_dict=None):

    prio_to_buff_p = new_sx_cos_port_prio_buff_t_p()
    try:
        for prio in prio_to_buff_dict:
            sx_cos_port_buff_t_arr_setitem(prio_to_buff_p.prio_to_buff, prio, prio_to_buff_dict[prio])

        rc = sx_api_cos_port_prio_buff_map_set(handle, SX_ACCESS_CMD_SET, port, prio_to_buff_p)
        assert rc == 0, "Failed to set pg, switch priority mapping"
    finally:
        delete_sx_cos_port_prio_buff_t_p(prio_to_buff_p)

# API to get SP to TC mapping


def cos_port_tc_prio_map_get(handle, log_port, prio_id):
    traffic_class_p = new_sx_cos_traffic_class_t_p()
    try:
        rc = sx_api_cos_port_tc_prio_map_get(handle, log_port, prio_id, traffic_class_p)
        assert rc == 0, "Failed to set pg, switch priority mapping"
        return sx_cos_traffic_class_t_p_value(traffic_class_p)
    finally:
        delete_sx_cos_traffic_class_t_p(traffic_class_p)


# API to set SP to TC mapping
def cos_port_prio_buff_map_to_tc(handle, log_port, prio_id, tc_buffer_id):
    rc = sx_api_cos_port_tc_prio_map_set(handle, SX_ACCESS_CMD_ADD, log_port, prio_id, tc_buffer_id)
    assert rc == 0, "Failed to set tc, switch priority mapping"


def init_drop_reason_groups():
    global l2_cb, router_cb, tunnel_cb, acl_cb, l1_cb, buffer_cb
    l2_cb = WjhDropCallbacks()
    l2_cb.drop_reason_group = WJH_DROP_REASON_GROUP_L2_E
    l2_cb.raw_cb.L2 = WjhL2DropRawCb(wjh_l2_drop_raw_cb)
    l2_cb.aggregate_cb.L2 = WjhL2DropAggregateCb(wjh_l2_drop_aggregate_cb)
    attr = c_uint32()
    rc = wjh_lib.wjh_drop_reason_group_init(WJH_DROP_REASON_GROUP_L2_E, byref(attr), byref(l2_cb))

    router_cb = WjhDropCallbacks()
    router_cb.drop_reason_group = WJH_DROP_REASON_GROUP_ROUTER_E
    router_cb.raw_cb.router = WjhRouterDropRawCb(wjh_router_drop_raw_cb)
    router_cb.aggregate_cb.router = WjhRouterDropAggregateCb(wjh_router_drop_aggregate_cb)
    attr = c_uint32()
    rc = wjh_lib.wjh_drop_reason_group_init(WJH_DROP_REASON_GROUP_ROUTER_E, byref(attr), byref(router_cb))

    tunnel_cb = WjhDropCallbacks()
    tunnel_cb.drop_reason_group = WJH_DROP_REASON_GROUP_TUNNEL_E
    tunnel_cb.raw_cb.tunnel = WjhTunnelDropRawCb(wjh_tunnel_drop_raw_cb)
    tunnel_cb.aggregate_cb.tunnel = WjhTunnelDropAggregateCb(wjh_tunnel_drop_aggregate_cb)
    attr = c_uint32()
    rc = wjh_lib.wjh_drop_reason_group_init(WJH_DROP_REASON_GROUP_TUNNEL_E, byref(attr), byref(tunnel_cb))

    acl_cb = WjhDropCallbacks()
    acl_cb.drop_reason_group = WJH_DROP_REASON_GROUP_ACL_E
    acl_cb.raw_cb.acl = WjhAclDropRawCb(wjh_acl_drop_raw_cb)
    acl_cb.aggregate_cb.acl = WjhAclDropAggregateCb(wjh_acl_drop_aggregate_cb)
    attr = c_uint32()
    rc = wjh_lib.wjh_drop_reason_group_init(WJH_DROP_REASON_GROUP_ACL_E, byref(attr), byref(acl_cb))

    buffer_cb = WjhDropCallbacks()
    buffer_cb.drop_reason_group = WJH_DROP_REASON_GROUP_BUFFER_E
    buffer_cb.raw_cb.buffer = WjhBufferDropRawCb(wjh_buffer_drop_raw_cb)
    buffer_cb.aggregate_cb.buffer = WjhBufferDropAggregateCb(wjh_buffer_drop_aggregate_cb)
    attr = WjhDropReasonGroupAttr()
    attr.attr.buffer_drop.basic_attr.recirculation_port = RECYCLE_PORT
    attr.attr.buffer_drop.advanced_attr.trap_probability = 8
    attr.max_aggregation_entries_cnt = 0
    configure_recirculation_port(attr.attr.buffer_drop.basic_attr.recirculation_port)
    rc = wjh_lib.wjh_drop_reason_group_init(WJH_DROP_REASON_GROUP_BUFFER_E, byref(attr), byref(buffer_cb))
    assert 0 == rc, "Failed to init buffer drop reason group"

    l1_cb = WjhDropCallbacks()
    l1_cb.drop_reason_group = WJH_DROP_REASON_GROUP_L1_E
    l1_cb.aggregate_cb.L1 = WjhL1DropAggregateCb(wjh_l1_drop_aggregate_cb)
    l1_cb.raw_cb.L1 = WjhL1DropRawCb(wjh_l1_drop_raw_cb)
    attr = c_uint32()
    rc = wjh_lib.wjh_drop_reason_group_init(WJH_DROP_REASON_GROUP_L1_E, byref(attr), byref(l1_cb))

    return rc


def create_channels(mode):
    global raw_channel, aggregation_channel, cyclic_and_aggregation_channel, channel_mode, channel_to_agg_read_mode

    raw_channel = create_user_channel(WJH_USER_CHANNEL_CYCLIC_E)
    aggregation_channel = create_user_channel(WJH_USER_CHANNEL_AGGREGATE_E)
    cyclic_and_aggregation_channel = create_user_channel(WJH_USER_CHANNEL_CYCLIC_AND_AGGREGATE_E)

    if (mode is WJH_USER_CHANNEL_MODE_PULL_E):
        channel_mode = WJH_USER_CHANNEL_MODE_PULL_E
        user_channel_attr = WjhUserChannelAttr()
        user_channel_attr.polling_interval = 0
        user_channel_attr.mode = WJH_USER_CHANNEL_MODE_PULL_E
        user_channel_attr.aggregation_read_mode = WJH_AGGREGATION_READ_MODE_READ_CLEAR

        rc = wjh_lib.wjh_user_channel_set(raw_channel, byref(user_channel_attr))
        rc = wjh_lib.wjh_user_channel_set(aggregation_channel, byref(user_channel_attr))
        rc = wjh_lib.wjh_user_channel_set(cyclic_and_aggregation_channel, byref(user_channel_attr))

    for channel in (raw_channel, aggregation_channel, cyclic_and_aggregation_channel):
        channel_to_agg_read_mode[channel] = WJH_AGGREGATION_READ_MODE_READ_CLEAR

    return raw_channel, aggregation_channel, cyclic_and_aggregation_channel


def destroy_channels():
    global raw_channel, aggregation_channel, cyclic_and_aggregation_channel
    rc = wjh_lib.wjh_user_channel_destroy(raw_channel)

    rc = wjh_lib.wjh_user_channel_destroy(aggregation_channel)
    rc = wjh_lib.wjh_user_channel_destroy(cyclic_and_aggregation_channel)


def wjh_init(bandwidth, aggregation_key_mode):
    global init_param
    init_param = WjhInitParam()
    init_param.force = 1
    init_param.deactive_cb = WjhDeactiveCb(wjh_deactive_cb)
    init_param.bandwidth_percent = bandwidth
    init_param.log_cb = 0
    init_param.conf_xml_path = 0
    init_param.debug_fs_path = 0
    init_param.ingress_info_type = WJH_INGRESS_INFO_TYPE_LOGPORT
    init_param.aggregation_key_mode = agg_key_mode_str_to_enum[aggregation_key_mode]

    rc = wjh_lib.wjh_init(byref(init_param))

    return rc


def wjh_deinit():
    rc = wjh_lib.wjh_deinit()
    return rc


def wjh_action_enable(bandwidth, mode_str, print_packet, save_packet, aggregation_key_mode):
    global enable_print, enable_save

    open_new_files()

    rc = wjh_init(bandwidth, aggregation_key_mode)
    assert 0 == rc, "Failed to init wjh service lib"

    mode = channel_mode_str_to_enum[mode_str]
    if mode is WJH_USER_CHANNEL_MODE_PUSH_E:
        enable_print = print_packet
        enable_save = save_packet
        if enable_save:
            print("Dropped packets will be saved to %s and aggregation statistics will be saved to %s" % (wjh_raw_file, wjh_agg_file))
    create_channels(mode)
    init_drop_reason_groups()

    configure_drop_reason_group(WJH_DROP_REASON_GROUP_L2_E)
    configure_drop_reason_group(WJH_DROP_REASON_GROUP_ROUTER_E)
    configure_drop_reason_group(WJH_DROP_REASON_GROUP_TUNNEL_E)
    configure_drop_reason_group(WJH_DROP_REASON_GROUP_ACL_E)
    configure_drop_reason_group(WJH_DROP_REASON_GROUP_BUFFER_E)
    configure_drop_reason_group(WJH_DROP_REASON_GROUP_L1_E)

    return rc, "WJH example - Backend started"


def wjh_action_print_save(raw_file_path, agg_file_path):
    global raw_channel, aggregation_channel, enable_print, enable_save, pcap_user_fd, txt_user_fd

    enable_print = True
    rc, msg = wjh_action_save(raw_file_path, agg_file_path)
    enable_print = False

    return rc, "WJH example - dropped packets printed and saved"


def wjh_action_print():
    global raw_channel, aggregation_channel, cyclic_and_aggregation_channel, enable_print

    enable_print = True
    rc = wjh_lib.wjh_user_channel_pull(raw_channel)
    rc = wjh_lib.wjh_user_channel_pull(aggregation_channel)
    rc = wjh_lib.wjh_user_channel_pull(cyclic_and_aggregation_channel)
    enable_print = False

    return rc, "WJH example - dropped packets printed"


def wjh_action_save(raw_file_path, agg_file_path):
    global raw_channel, aggregation_channel, cyclic_and_aggregation_channel, enable_save, pcap_user_fd, txt_user_fd, wjh_raw_file, wjh_agg_file

    enable_save = True

    new_pcap = False
    if raw_file_path:
        if os.path.isfile(raw_file_path):
            pcap_user_fd = open(raw_file_path, "a")
        else:
            pcap_user_fd = open(raw_file_path, "w")
            new_pcap = True
        if pcap_user_fd < 0:
            return -1, "WJH example - failed to open raw file to save"
        if new_pcap:
            prepare_pcap(pcap_user_fd)
        wjh_raw_file = raw_file_path

    if agg_file_path:
        if os.path.isfile(agg_file_path):
            txt_user_fd = open(agg_file_path, "a")
        else:
            txt_user_fd = open(agg_file_path, "w")
        if txt_user_fd < 0:
            return -1, "WJH example - failed to open raw file to save"
        wjh_agg_file = agg_file_path

    rc = wjh_lib.wjh_user_channel_pull(raw_channel)
    rc = wjh_lib.wjh_user_channel_pull(aggregation_channel)
    rc = wjh_lib.wjh_user_channel_pull(cyclic_and_aggregation_channel)

    print("Dropped packets saved to %s and aggregation statistics saved to %s" % (wjh_raw_file, wjh_agg_file))

    if pcap_user_fd >= 0:
        pcap_user_fd.close()
        pcap_user_fd = -1
    if txt_user_fd >= 0:
        txt_user_fd.close()
        txt_user_fd = -1

    enable_save = False
    wjh_raw_file = WJH_PCAP_NAME
    wjh_agg_file = WJH_TXT_NAME

    return rc, "WJH example - dropped packets saved"


def wjh_action_enable_group(drop_reason_group_str, severity_str):
    drop_reason_group = drop_reason_group_str_to_enum[drop_reason_group_str]
    severity = severity_str_to_enum[severity_str]
    if drop_reason_group is WJH_DROP_REASON_GROUP_ALL_E:
        for i in range(WJH_DROP_REASON_GROUP_BUFFER_E, WJH_DROP_REASON_GROUP_ALL_E):
            rc = wjh_lib.wjh_drop_reason_group_enable(i, severity)
    else:
        rc = wjh_lib.wjh_drop_reason_group_enable(drop_reason_group, severity)

    return rc, "WJH example - drop reason group enabled"


def wjh_action_disable_group(drop_reason_group_str, severity_str):
    drop_reason_group = drop_reason_group_str_to_enum[drop_reason_group_str]
    severity = severity_str_to_enum[severity_str]
    if drop_reason_group is WJH_DROP_REASON_GROUP_ALL_E:
        for i in range(WJH_DROP_REASON_GROUP_BUFFER_E, WJH_DROP_REASON_GROUP_ALL_E):
            rc = wjh_lib.wjh_drop_reason_group_disable(i, severity)
    else:
        rc = wjh_lib.wjh_drop_reason_group_disable(drop_reason_group, severity)

    return rc, "WJH example - drop reason group disabled"


def wjh_action_bind_to_channel_impl(drop_reason_group, channel_type):
    drop_reason_group_str = drop_reason_group_to_str[drop_reason_group]
    channel_id = 0
    if channel_type is WJH_USER_CHANNEL_CYCLIC_E:
        channel_id = raw_channel
    elif channel_type is WJH_USER_CHANNEL_AGGREGATE_E:
        channel_id = aggregation_channel
    else:
        channel_id = cyclic_and_aggregation_channel
    if drop_reason_group_to_channel[drop_reason_group] == channel_id:
        return 0, "WJH example - drop reason group %s is already bound to %s channel" % (drop_reason_group_str, channel_type_to_str[channel_type])
    rc = wjh_lib.wjh_drop_reason_group_unbind(drop_reason_group)
    if rc != 0:
        return rc, "WJH example - failed to unbind drop reason group %s" % (drop_reason_group_str)
    rc = wjh_lib.wjh_drop_reason_group_bind(drop_reason_group, channel_id)
    if rc != 0:
        return rc, "WJH example - failed to bind drop reason group %s to raw channel" % (drop_reason_group_str)
    drop_reason_group_to_channel[drop_reason_group] = channel_id
    rc = wjh_lib.wjh_drop_reason_group_enable(drop_reason_group, WJH_SEVERITY_ALL_E)
    if rc != 0:
        return rc, "WJH example - failed to enable severity \"all\" for drop reason group %s" % (drop_reason_group_str)
    return rc, "WJH example - drop reason group %s is bound to %s channel, severity \"all\" is enabled for it" % (drop_reason_group_str, channel_type_to_str[channel_type])


def wjh_action_bind_to_channel(drop_reason_group_str, channel_type):
    rc = 0
    str = ""
    drop_reason_group = drop_reason_group_str_to_enum[drop_reason_group_str]
    if drop_reason_group is WJH_DROP_REASON_GROUP_ALL_E:
        for i in range(WJH_DROP_REASON_GROUP_BUFFER_E, WJH_DROP_REASON_GROUP_ALL_E):
            rc1, str1 = wjh_action_bind_to_channel_impl(i, channel_type)
            if rc1 != 0:
                rc = rc1
                str = str1
                break
            else:
                str += str1
                str += "\n"
    else:
        rc1, str1 = wjh_action_bind_to_channel_impl(drop_reason_group, channel_type)
        if rc1 != 0:
            rc = rc1
            str = str1
        else:
            str += str1
            str += "\n"

    if rc == 0:
        str += "Successfully bound drop reason group %s to %s channel" % (drop_reason_group_str, channel_type_to_str[channel_type])

    return rc, str


def wjh_action_set_channel_aggregation_read_mode(channel_id, agg_read_mode_str):
    global channel_mode, channel_to_agg_read_mode

    user_channel_attr = WjhUserChannelAttr()
    user_channel_attr.polling_interval = 0
    user_channel_attr.mode = channel_mode
    user_channel_attr.aggregation_read_mode = agg_read_mode_str_to_enum[agg_read_mode_str]

    rc = wjh_lib.wjh_user_channel_set(channel_id, byref(user_channel_attr))
    if rc != 0:
        return rc, "WJH example - failed to set user channel %d aggregation read mode to %s" % (channel_id, agg_read_mode_str)

    channel_to_agg_read_mode[channel_id] = agg_read_mode_str_to_enum[agg_read_mode_str]

    return rc, "WJH example - successfully set user channel %d aggregation read mode to %s" % (channel_id, agg_read_mode_str)


def wjh_action_set_aggregation_channel_aggregation_read_mode(agg_read_mode_str):
    global aggregation_channel
    return wjh_action_set_channel_aggregation_read_mode(aggregation_channel, agg_read_mode_str)


def wjh_action_set_cyclic_and_aggregation_channel_aggregation_read_mode(agg_read_mode_str):
    global cyclic_and_aggregation_channel
    return wjh_action_set_channel_aggregation_read_mode(cyclic_and_aggregation_channel, agg_read_mode_str)


def wjh_action_set_channel_timestamp_source(channel_timestamp_source_str):
    global channel_mode, channel_to_agg_read_mode
    ret_str = ""

    for channel in (raw_channel, aggregation_channel, cyclic_and_aggregation_channel):
        user_channel_attr = WjhUserChannelAttr()
        user_channel_attr.polling_interval = 0
        user_channel_attr.mode = channel_mode
        user_channel_attr.aggregation_read_mode = channel_to_agg_read_mode[channel]
        user_channel_attr.timestamp_source = channel_timestamp_source_str_to_enum[channel_timestamp_source_str]

        rc = wjh_lib.wjh_user_channel_set(channel, byref(user_channel_attr))
        if rc != 0:
            return rc, "WJH example - failed to set user channel %d timestamp source to %s" % (channel, channel_timestamp_source_str)

        ret_str += "WJH example - successfully set user channel %d timestamp source to %s\n" % (channel, channel_timestamp_source_str)

    return rc, ret_str


def wjh_action_disable():
    global pcap_fd, txt_fd
    rc = 0

    deconfigure_drop_reason_group(WJH_DROP_REASON_GROUP_ACL_E)
    deconfigure_drop_reason_group(WJH_DROP_REASON_GROUP_BUFFER_E)
    deconfigure_drop_reason_group(WJH_DROP_REASON_GROUP_L1_E)
    deconfigure_drop_reason_group(WJH_DROP_REASON_GROUP_L2_E)
    deconfigure_drop_reason_group(WJH_DROP_REASON_GROUP_ROUTER_E)
    deconfigure_drop_reason_group(WJH_DROP_REASON_GROUP_TUNNEL_E)
    deconfigure_recirculation_port(RECYCLE_PORT)

    destroy_channels()

    wjh_deinit()

    pcap_fd.close()
    txt_fd.close()

    return rc, "WJH example - Backend stopped"


def wjh_receive_signal(signum, frame):
    print("WJH example - Backend stopped")
    wjh_action_disable()
    sys.exit(0)

# API to send message to wjh tool client


def wjh_send_client_msg(conn, msg, rc):
    if rc != 0:
        msg = msg + " rc:" + str(rc)
    try:
        conn.sendall(msg)
        rval = True
    except Exception:
        print("Stopping WJH monitor exit: Unable to send message")
        rval = False
    return rval


def bandwidth_type(x):
    x = int(x)
    if x < 0 or x > 100:
        raise argparse.ArgumentTypeError("Bandwidth is 0 - 100")
    return x


# API to wait for incoming connection and messages from tool front end
def wjh_wait_incoming_connection():
    parser = argparse.ArgumentParser(description='WJH example Backend')
    parser.add_argument('--disable', action='store_true', help='disables the WJH example backend')
    parser.add_argument('--enable', action='store_true', help='enables the WJH example backend')
    parser.add_argument('--bandwidth', default=0, type=bandwidth_type, help='set pcie bandwidth percentage for WJH for enable option')
    parser.add_argument('--mode', default='manual', choices=['agent', 'manual'], help='set WJH mode for enable option')
    parser.add_argument('--aggregation_key_mode', default='streaming', choices=['streaming', 'detailed'], help='set WJH aggregation key mode for enable action')
    parser.add_argument('--print', action='store_true', help='prints the WJH captured packets')
    parser.add_argument('--save', action='store_true', help='saves the WJH captured packets')
    parser.add_argument('--raw_file', help='the optional file full path of the WJH raw packets')
    parser.add_argument('--agg_file', help='the optional file full path of the WJH aggregation info')
    parser.add_argument('--enable_drop_reason_group',
                        choices=['all', 'l1', 'l2', 'router', 'tunnel', 'acl', 'buffer'],
                        help='enable drop reason group')
    parser.add_argument('--disable_drop_reason_group',
                        choices=['all', 'l1', 'l2', 'router', 'tunnel', 'acl', 'buffer'],
                        help='disable drop reason group')
    parser.add_argument('--severity', default='all',
                        choices=['all', 'notice', 'warning', 'error'],
                        help='drop reason severity for drop reason group disable/enable')
    parser.add_argument('--bind_to_aggregation_channel',
                        choices=['all', 'l1', 'l2', 'router', 'tunnel', 'acl', 'buffer'],
                        help='bind drop reason group to aggregation channel')
    parser.add_argument('--bind_to_cyclic_channel',
                        choices=['all', 'l1', 'l2', 'router', 'tunnel', 'acl', 'buffer'],
                        help='bind drop reason group to cyclic channel')
    parser.add_argument('--bind_to_cyclic_and_aggregation_channel',
                        choices=['all', 'l1', 'l2', 'router', 'tunnel', 'acl', 'buffer'],
                        help='bind drop reason group to cyclic-and-aggregation channel')
    parser.add_argument('--set_aggregation_channel_aggregation_read_mode',
                        choices=['read_clear', 'read'],
                        help='set the aggregation read mode of the aggregation channel')
    parser.add_argument('--set_cyclic_and_aggregation_channel_aggregation_read_mode',
                        choices=['read_clear', 'read'],
                        help='set the aggregation read mode of the cyclic-and-aggregation channel')
    parser.add_argument('--set_channel_timestamp_source',
                        choices=['linux', 'hw_clock'],
                        help='set the channel timestamp source')

    is_monitoring = False
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        server_address = ('localhost', WJH_MONITOR_SERVER_PORT)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.bind(server_address)
        sock.listen(WJH_SERVER_LISTEN_QUEUE_SIZE)
    except socket.error:
        print("WJH example backend already running.")
        enabled = False
    except Exception:
        print("WJH example backend- Failed.")
        enabled = False
    else:
        enabled = True

        while enabled:
            try:
                connection, client_address = sock.accept()
                recv_data = connection.recv(WJH_SOCKET_BUFFER_SIZE)
                cmd = pickle.loads(recv_data)
                args = parser.parse_args(cmd)

                # handle --enable
                if args.enable:
                    if is_monitoring:
                        rval = wjh_send_client_msg(connection, "WJH example backend - Monitoring is already ON", 0)
                        enabled = rval
                    else:
                        print_packet = False
                        save_packet = False
                        if vars(args)['print']:
                            print_packet = True
                        if args.save:
                            save_packet = True
                        rc, msg = wjh_action_enable(args.bandwidth, args.mode, print_packet, save_packet, args.aggregation_key_mode)
                        rval = wjh_send_client_msg(connection, msg, rc)
                        # start failure would result in agent termination
                        if rc == 0:
                            enabled = rval
                            is_monitoring = True
                        else:
                            enabled = False
                            is_monitoring = False
                # handle --disable
                elif args.disable:
                    enabled = False
                    rc, msg = wjh_action_disable()
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --print and --save:
                elif vars(args)['print'] and args.save:
                    rc, msg = wjh_action_print_save(args.raw_file, args.agg_file)
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --print
                elif vars(args)['print']:
                    rc, msg = wjh_action_print()
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --save
                elif args.save:
                    rc, msg = wjh_action_save(args.raw_file, args.agg_file)
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --enable_drop_reason_group
                elif args.enable_drop_reason_group:
                    rc, msg = wjh_action_enable_group(args.enable_drop_reason_group, args.severity)
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --disable_drop_reason_group
                elif args.disable_drop_reason_group:
                    rc, msg = wjh_action_disable_group(args.disable_drop_reason_group, args.severity)
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --bind_to_aggregation_channel
                elif args.bind_to_aggregation_channel:
                    rc, msg = wjh_action_bind_to_channel(args.bind_to_aggregation_channel, WJH_USER_CHANNEL_AGGREGATE_E)
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --bind_to_cyclic_channel
                elif args.bind_to_cyclic_channel:
                    rc, msg = wjh_action_bind_to_channel(args.bind_to_cyclic_channel, WJH_USER_CHANNEL_CYCLIC_E)
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --bind_to_cyclic_and_aggregation_channel
                elif args.bind_to_cyclic_and_aggregation_channel:
                    rc, msg = wjh_action_bind_to_channel(args.bind_to_cyclic_and_aggregation_channel, WJH_USER_CHANNEL_CYCLIC_AND_AGGREGATE_E)
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --set_aggregation_channel_aggregation_read_mode
                elif args.set_aggregation_channel_aggregation_read_mode:
                    rc, msg = wjh_action_set_aggregation_channel_aggregation_read_mode(args.set_aggregation_channel_aggregation_read_mode)
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --set_cyclic_and_aggregation_channel_aggregation_read_mode
                elif args.set_cyclic_and_aggregation_channel_aggregation_read_mode:
                    rc, msg = wjh_action_set_cyclic_and_aggregation_channel_aggregation_read_mode(args.set_cyclic_and_aggregation_channel_aggregation_read_mode)
                    rval = wjh_send_client_msg(connection, msg, rc)
                # handle --set_channel_timestamp_source
                elif args.set_channel_timestamp_source:
                    rc, msg = wjh_action_set_channel_timestamp_source(args.set_channel_timestamp_source)
                    rval = wjh_send_client_msg(connection, msg, rc)
                else:
                    rval = wjh_send_client_msg(connection, "WJH example backend- Invalid command", 0)
                    enabled = rval

                connection.close()
            except socket.error:
                print("WJH example backend - Unable to accept or recv data from new connection start again")
                enabled = False
                is_monitoring = False
            except Exception:
                print("WJH example backend - Failure occured , exiting..")
                enabled = False
                is_monitoring = False
    finally:
        sock.close()
        if enabled:
            wjh_action_disable()

#######################################################################################
###################################BUFFER FUNCTIONS####################################
#######################################################################################


# Dictionaries used by buffer API's
reserved_buffer_types_to_field_names_dict = {
    SX_COS_INGRESS_PORT_ATTR_E: "ingress_port_buff_attr",
    SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E: "ingress_port_pg_buff_attr",
    SX_COS_EGRESS_PORT_ATTR_E: "egress_port_buff_attr",
    SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E: "egress_port_tc_buff_attr",
    SX_COS_MULTICAST_ATTR_E: "multicast_buff_attr",
    SX_COS_MULTICAST_PORT_ATTR_E: "multicast_port_buff_attr",
    SX_COS_PORT_BUFF_ATTR_RESERVED1_E: "ingress_port_buff_attr",
    SX_COS_PORT_BUFF_ATTR_RESERVED2_E: "ingress_port_pg_buff_attr",
    SX_COS_PORT_BUFF_ATTR_RESERVED3_E: "egress_port_buff_attr",
    SX_COS_PORT_BUFF_ATTR_RESERVED4_E: "egress_port_tc_buff_attr"
}

buffer_types_to_buffer_id_names_dict = {
    SX_COS_INGRESS_PORT_ATTR_E: "pool_id",
    SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E: "pg",
    SX_COS_EGRESS_PORT_ATTR_E: "pool_id",
    SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E: "tc",
    SX_COS_MULTICAST_ATTR_E: "sp",
    SX_COS_MULTICAST_PORT_ATTR_E: "pool_id",
    SX_COS_PORT_BUFF_ATTR_RESERVED1_E: "pool_id",
    SX_COS_PORT_BUFF_ATTR_RESERVED2_E: "pg",
    SX_COS_PORT_BUFF_ATTR_RESERVED3_E: "pool_id",
    SX_COS_PORT_BUFF_ATTR_RESERVED4_E: "tc"
}

shared_buffer_types_to_field_names_dict = {
    SX_COS_INGRESS_PORT_ATTR_E: "ingress_port_shared_buff_attr",
    SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E: "ingress_port_pg_shared_buff_attr",
    SX_COS_EGRESS_PORT_ATTR_E: "egress_port_shared_buff_attr",
    SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E: "egress_port_tc_shared_buff_attr",
    SX_COS_MULTICAST_ATTR_E: "multicast_shared_buff_attr",
    SX_COS_MULTICAST_PORT_ATTR_E: "multicast_port_shared_buff_attr",
    SX_COS_PORT_BUFF_ATTR_RESERVED1_E: "ingress_port_shared_buff_attr",
    SX_COS_PORT_BUFF_ATTR_RESERVED2_E: "ingress_port_pg_shared_buff_attr",
    SX_COS_PORT_BUFF_ATTR_RESERVED3_E: "egress_port_shared_buff_attr",
    SX_COS_PORT_BUFF_ATTR_RESERVED4_E: "egress_port_tc_shared_buff_attr"
}

# API to change the reserved buffer settings.


def modify_port_reserved_buffer(handle, cmd, log_port, type, buffer_id, size=0, pool_id=0, is_lossy=1,
                                xon=0, xoff=0, count=1, pipeline_latency_override=0, pipeline_latency_size=0,
                                attr_list=None):

    port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(count)
    attr_item = new_sx_cos_port_buffer_attr_t_p()
    try:
        if attr_list is None:
            if type > SX_COS_PORT_BUFF_ATTR_RESERVED4_E or type < SX_COS_INGRESS_PORT_ATTR_E or \
                    type in [SX_COS_INGRESS_PORT_PRIORITY_GROUP_PIPELINE_LATENCY_ATTR_E, SX_COS_INGRESS_PORT_PRIORITY_GROUP_HEADROOM_ATTR_E]:
                return (-1)

            attr_item.type = type
            attr = getattr(attr_item.attr, reserved_buffer_types_to_field_names_dict[type])
            attr.size = size
            if type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
                attr.is_lossy = is_lossy
                attr.xon = xon
                attr.xoff = xoff
                attr.pipeline_latency.override_default = pipeline_latency_override
                attr.pipeline_latency.size = pipeline_latency_size
            setattr(attr, buffer_types_to_buffer_id_names_dict[type], buffer_id)

            sx_cos_port_buffer_attr_t_arr_setitem(port_buffer_attr_list_p, 0, attr_item)

        else:
            for i in xrange(count):
                sx_cos_port_buffer_attr_t_arr_setitem(port_buffer_attr_list_p, i, attr_list[i])

        rc = sx_api_cos_port_buff_type_set(handle, cmd, log_port, port_buffer_attr_list_p, count)
        assert 0 == rc, "Failed to set port reserved buffer"
    finally:
        delete_sx_cos_port_buffer_attr_t_p(attr_item)
        delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)


# API to change the shared buffer settings.
def modify_port_shared_buffer(handle, cmd, log_port, type, buffer_id, pool_id=0, mode=0, size=0, alpha=0,
                              count=1, attr_list=None):
    port_shared_buffer_attr_list_p = new_sx_cos_port_shared_buffer_attr_t_arr(count)
    attr_item = new_sx_cos_port_shared_buffer_attr_t_p()
    try:
        if attr_list is None:
            if type > SX_COS_PORT_BUFF_ATTR_RESERVED4_E or type < SX_COS_INGRESS_PORT_ATTR_E or \
                    type in [SX_COS_INGRESS_PORT_PRIORITY_GROUP_PIPELINE_LATENCY_ATTR_E, SX_COS_INGRESS_PORT_PRIORITY_GROUP_HEADROOM_ATTR_E]:
                return (-1)

            attr_item.type = type
            attr = getattr(attr_item.attr, shared_buffer_types_to_field_names_dict[type])

            attr.pool_id = pool_id  # this has effect only for non-pool buffers.
            attr.max.mode = mode
            if mode in [SX_COS_BUFFER_MAX_MODE_STATIC_E, SX_COS_BUFFER_MAX_MODE_BUFFER_UNITS_E]:
                attr.max.max.size = size
            elif mode == SX_COS_BUFFER_MAX_MODE_DYNAMIC_E:
                attr.max.max.alpha = alpha
            setattr(attr, buffer_types_to_buffer_id_names_dict[type], buffer_id)

            sx_cos_port_shared_buffer_attr_t_arr_setitem(port_shared_buffer_attr_list_p, 0, attr_item)

        else:
            for i in xrange(count):
                sx_cos_port_shared_buffer_attr_t_arr_setitem(port_shared_buffer_attr_list_p, i, attr_list[i])

        rc = sx_api_cos_port_shared_buff_type_set(handle, cmd, log_port, port_shared_buffer_attr_list_p, count)
        assert 0 == rc, "Failed to set port shared buffer"
    finally:
        delete_sx_cos_port_shared_buffer_attr_t_p(attr_item)
        delete_sx_cos_port_shared_buffer_attr_t_arr(port_shared_buffer_attr_list_p)

# API to get the reserved buffer attributes


def get_reserved_buffer_attributes(handle, log_port, type, value):

    if type > SX_COS_PORT_BUFF_ATTR_RESERVED4_E or type < SX_COS_INGRESS_PORT_ATTR_E or \
        type in [SX_COS_INGRESS_PORT_PRIORITY_GROUP_PIPELINE_LATENCY_ATTR_E,
                 SX_COS_INGRESS_PORT_PRIORITY_GROUP_HEADROOM_ATTR_E]:
        return(-1)

    port_buffer_attr_cnt = copy_uint32_t_p(1)
    port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(1)

    try:
        port_buffer_attr_item = sx_cos_port_buffer_attr_t()
        port_buffer_attr_item.type = type
        attr = getattr(port_buffer_attr_item.attr, reserved_buffer_types_to_field_names_dict[type])
        setattr(attr, buffer_types_to_buffer_id_names_dict[type], value)

        sx_cos_port_buffer_attr_t_arr_setitem(port_buffer_attr_list_p, 0, port_buffer_attr_item)

        rc = sx_api_cos_port_buff_type_get(handle, log_port, port_buffer_attr_list_p, port_buffer_attr_cnt)
        assert 0 == rc, "Failed to get port reserved buffer"

        return sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, 0)
    finally:
        delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)
        delete_uint32_t_p(port_buffer_attr_cnt)


# API to get the shared  buffer attributes
def get_shared_buffer_attributes(handle, log_port, type, value):

    if type > SX_COS_PORT_BUFF_ATTR_RESERVED4_E or type < SX_COS_INGRESS_PORT_ATTR_E or \
        type in [SX_COS_INGRESS_PORT_PRIORITY_GROUP_PIPELINE_LATENCY_ATTR_E,
                 SX_COS_INGRESS_PORT_PRIORITY_GROUP_HEADROOM_ATTR_E]:
        return(-1)

    port_shared_buffer_attr_cnt = copy_uint32_t_p(1)
    port_shared_buffer_attr_list_p = new_sx_cos_port_shared_buffer_attr_t_arr(1)

    try:
        port_shared_buffer_attr_item = sx_cos_port_shared_buffer_attr_t()
        port_shared_buffer_attr_item.type = type

        attr = getattr(port_shared_buffer_attr_item.attr, shared_buffer_types_to_field_names_dict[type])
        setattr(attr, buffer_types_to_buffer_id_names_dict[type], value)

        sx_cos_port_shared_buffer_attr_t_arr_setitem(port_shared_buffer_attr_list_p, 0, port_shared_buffer_attr_item)

        rc = sx_api_cos_port_shared_buff_type_get(handle, log_port, port_shared_buffer_attr_list_p,
                                                  port_shared_buffer_attr_cnt)
        assert 0 == rc, "Failed to get port reserved buffer"
        return sx_cos_port_shared_buffer_attr_t_arr_getitem(port_shared_buffer_attr_list_p, 0)
    finally:
        delete_sx_cos_port_shared_buffer_attr_t_arr(port_shared_buffer_attr_list_p)
        delete_uint32_t_p(port_shared_buffer_attr_cnt)

#######################################################################################
###############################BUFFER API END##########################################
#######################################################################################


def main():
    global handle
    rc, handle = sx_api_open(None)
    if (rc != SX_STATUS_SUCCESS):
        print ("Failed to open api handle.\nPlease check that SDK is running.")
    wjh_wait_incoming_connection()


signal.signal(signal.SIGTERM, wjh_receive_signal)
signal.signal(signal.SIGINT, wjh_receive_signal)

ctypes.CDLL('libsxapi.so', mode=ctypes.RTLD_GLOBAL)
ctypes.CDLL('libsxadviser.so', mode=ctypes.RTLD_GLOBAL)
ctypes.CDLL('libsxaclhelper.so', mode=ctypes.RTLD_GLOBAL)
ctypes.CDLL('libscew.so', mode=ctypes.RTLD_GLOBAL)
try:
    wjh_lib = ctypes.CDLL('libwjh.so', mode=ctypes.RTLD_GLOBAL)
except OSError as e:
    ctypes.CDLL('libelf.so', mode=ctypes.RTLD_GLOBAL)
    wjh_lib = ctypes.CDLL('libwjh.so', mode=ctypes.RTLD_GLOBAL)


if __name__ == "__main__":
    main()
